# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'kikuchiUI.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets
import os
fn = os.path.join(os.path.dirname(__file__), "kikuchi-icon.png")


class Ui_Kikuchi(object):
    def setupUi(self, Kikuchi):
        Kikuchi.setObjectName("Kikuchi")
        Kikuchi.resize(1213, 783)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(fn), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Kikuchi.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(Kikuchi)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.groupBox_2 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_2.setMaximumSize(QtCore.QSize(350, 16777215))
        self.groupBox_2.setTitle("")
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_9 = QtWidgets.QGroupBox(self.groupBox_2)
        self.groupBox_9.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.groupBox_9.setObjectName("groupBox_9")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox_9)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.Button_addband = QtWidgets.QPushButton(self.groupBox_9)
        self.Button_addband.setEnabled(True)
        self.Button_addband.setObjectName("Button_addband")
        self.gridLayout_10.addWidget(self.Button_addband, 2, 3, 1, 1)
        self.ListBox_d_2 = QtWidgets.QListWidget(self.groupBox_9)
        self.ListBox_d_2.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ListBox_d_2.sizePolicy().hasHeightForWidth())
        self.ListBox_d_2.setSizePolicy(sizePolicy)
        self.ListBox_d_2.setMaximumSize(QtCore.QSize(16777215, 100))
        self.ListBox_d_2.setObjectName("ListBox_d_2")
        self.gridLayout_10.addWidget(self.ListBox_d_2, 2, 0, 2, 3)
        self.Button_remove_band = QtWidgets.QPushButton(self.groupBox_9)
        self.Button_remove_band.setObjectName("Button_remove_band")
        self.gridLayout_10.addWidget(self.Button_remove_band, 3, 3, 1, 1)
        self.ListBox_theo = QtWidgets.QListWidget(self.groupBox_9)
        self.ListBox_theo.setObjectName("ListBox_theo")
        self.gridLayout_10.addWidget(self.ListBox_theo, 13, 0, 2, 4)
        self.plot_bands_button = QtWidgets.QPushButton(self.groupBox_9)
        self.plot_bands_button.setObjectName("plot_bands_button")
        self.gridLayout_10.addWidget(self.plot_bands_button, 4, 3, 1, 1)
        self.Button_add_center = QtWidgets.QPushButton(self.groupBox_9)
        self.Button_add_center.setObjectName("Button_add_center")
        self.gridLayout_10.addWidget(self.Button_add_center, 0, 0, 1, 2)
        self.centerLine = QtWidgets.QLineEdit(self.groupBox_9)
        self.centerLine.setObjectName("centerLine")
        self.gridLayout_10.addWidget(self.centerLine, 0, 2, 1, 2)
        self.label = QtWidgets.QLabel(self.groupBox_9)
        self.label.setObjectName("label")
        self.gridLayout_10.addWidget(self.label, 4, 0, 1, 1)
        self.tol_entry = QtWidgets.QLineEdit(self.groupBox_9)
        self.tol_entry.setObjectName("tol_entry")
        self.gridLayout_10.addWidget(self.tol_entry, 4, 1, 1, 1)
        self.Button_run = QtWidgets.QPushButton(self.groupBox_9)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.Button_run.setFont(font)
        self.Button_run.setObjectName("Button_run")
        self.gridLayout_10.addWidget(self.Button_run, 12, 0, 1, 4)
        self.Button_orientation = QtWidgets.QPushButton(self.groupBox_9)
        self.Button_orientation.setObjectName("Button_orientation")
        self.gridLayout_10.addWidget(self.Button_orientation, 15, 0, 1, 4)
        self.gridLayout.addWidget(self.groupBox_9, 2, 0, 2, 5)
        self.groupBox = QtWidgets.QGroupBox(self.groupBox_2)
        self.groupBox.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setContentsMargins(5, 5, 5, 5)
        self.gridLayout_2.setSpacing(5)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.Button_reset = QtWidgets.QPushButton(self.groupBox)
        self.Button_reset.setObjectName("Button_reset")
        self.gridLayout_2.addWidget(self.Button_reset, 25, 0, 1, 6)
        self.a_label = QtWidgets.QLabel(self.groupBox)
        self.a_label.setObjectName("a_label")
        self.gridLayout_2.addWidget(self.a_label, 0, 0, 1, 1)
        self.abc_entry = QtWidgets.QLineEdit(self.groupBox)
        self.abc_entry.setObjectName("abc_entry")
        self.gridLayout_2.addWidget(self.abc_entry, 0, 2, 1, 1)
        self.d_label_var = QtWidgets.QLabel(self.groupBox)
        self.d_label_var.setText("")
        self.d_label_var.setObjectName("d_label_var")
        self.gridLayout_2.addWidget(self.d_label_var, 9, 5, 1, 1)
        self.alpha_label = QtWidgets.QLabel(self.groupBox)
        self.alpha_label.setObjectName("alpha_label")
        self.gridLayout_2.addWidget(self.alpha_label, 9, 0, 1, 1)
        self.indice_label = QtWidgets.QLabel(self.groupBox)
        self.indice_label.setObjectName("indice_label")
        self.gridLayout_2.addWidget(self.indice_label, 14, 0, 1, 1)
        self.space_group_label = QtWidgets.QLabel(self.groupBox)
        self.space_group_label.setObjectName("space_group_label")
        self.gridLayout_2.addWidget(self.space_group_label, 15, 0, 1, 1)
        self.brightness_label = QtWidgets.QLabel(self.groupBox)
        self.brightness_label.setObjectName("brightness_label")
        self.gridLayout_2.addWidget(self.brightness_label, 16, 0, 1, 1)
        self.alphabetagamma_entry = QtWidgets.QLineEdit(self.groupBox)
        self.alphabetagamma_entry.setObjectName("alphabetagamma_entry")
        self.gridLayout_2.addWidget(self.alphabetagamma_entry, 9, 2, 1, 1)
        self.indice_entry = QtWidgets.QLineEdit(self.groupBox)
        self.indice_entry.setObjectName("indice_entry")
        self.gridLayout_2.addWidget(self.indice_entry, 14, 2, 1, 1)
        self.space_group_Box = QtWidgets.QComboBox(self.groupBox)
        self.space_group_Box.setObjectName("space_group_Box")
        self.gridLayout_2.addWidget(self.space_group_Box, 15, 2, 1, 1)
        self.brightness_slider = QtWidgets.QSlider(self.groupBox)
        self.brightness_slider.setOrientation(QtCore.Qt.Horizontal)
        self.brightness_slider.setObjectName("brightness_slider")
        self.gridLayout_2.addWidget(self.brightness_slider, 16, 2, 1, 1)
        self.Calib_label = QtWidgets.QLabel(self.groupBox)
        self.Calib_label.setObjectName("Calib_label")
        self.gridLayout_2.addWidget(self.Calib_label, 19, 0, 1, 1)
        self.Calib_box = QtWidgets.QComboBox(self.groupBox)
        self.Calib_box.setObjectName("Calib_box")
        self.gridLayout_2.addWidget(self.Calib_box, 19, 2, 1, 1)
        self.reset_view_button = QtWidgets.QPushButton(self.groupBox)
        self.reset_view_button.setObjectName("reset_view_button")
        self.gridLayout_2.addWidget(self.reset_view_button, 26, 0, 1, 6)
        self.gridLayout.addWidget(self.groupBox, 1, 0, 1, 1)
        self.gridLayout_7.addWidget(self.groupBox_2, 0, 2, 1, 1)
        self.mplvl = QtWidgets.QGridLayout()
        self.mplvl.setObjectName("mplvl")
        self.gridLayout_7.addLayout(self.mplvl, 0, 1, 1, 1)
        Kikuchi.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(Kikuchi)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1213, 22))
        self.menubar.setObjectName("menubar")
        self.menuSave = QtWidgets.QMenu(self.menubar)
        self.menuSave.setObjectName("menuSave")
        self.menuStructure = QtWidgets.QMenu(self.menubar)
        self.menuStructure.setObjectName("menuStructure")
        self.menuRefine = QtWidgets.QMenu(self.menubar)
        self.menuRefine.setObjectName("menuRefine")
        Kikuchi.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(Kikuchi)
        self.statusbar.setObjectName("statusbar")
        Kikuchi.setStatusBar(self.statusbar)
        self.actionSave_figure = QtWidgets.QAction(Kikuchi)
        self.actionSave_figure.setObjectName("actionSave_figure")
        self.actionCalculate_Schmid_factor = QtWidgets.QAction(Kikuchi)
        self.actionCalculate_Schmid_factor.setObjectName("actionCalculate_Schmid_factor")
        self.actionCalculate_spectrum = QtWidgets.QAction(Kikuchi)
        self.actionCalculate_spectrum.setObjectName("actionCalculate_spectrum")
        self.actionRefine_orientation = QtWidgets.QAction(Kikuchi)
        self.actionRefine_orientation.setObjectName("actionRefine_orientation")
        self.actionGet_orientation_at_zero_tilt = QtWidgets.QAction(Kikuchi)
        self.actionGet_orientation_at_zero_tilt.setObjectName("actionGet_orientation_at_zero_tilt")
        self.actionTilt = QtWidgets.QAction(Kikuchi)
        self.actionTilt.setObjectName("actionTilt")
        self.menuSave.addAction(self.actionSave_figure)
        self.menuRefine.addAction(self.actionRefine_orientation)
        self.menuRefine.addAction(self.actionTilt)
        self.menubar.addAction(self.menuSave.menuAction())
        self.menubar.addAction(self.menuStructure.menuAction())
        self.menubar.addAction(self.menuRefine.menuAction())

        self.retranslateUi(Kikuchi)
        QtCore.QMetaObject.connectSlotsByName(Kikuchi)
        Kikuchi.setTabOrder(self.abc_entry, self.alphabetagamma_entry)
        Kikuchi.setTabOrder(self.alphabetagamma_entry, self.indice_entry)
        Kikuchi.setTabOrder(self.indice_entry, self.space_group_Box)
        Kikuchi.setTabOrder(self.space_group_Box, self.brightness_slider)
        Kikuchi.setTabOrder(self.brightness_slider, self.Calib_box)
        Kikuchi.setTabOrder(self.Calib_box, self.Button_reset)
        Kikuchi.setTabOrder(self.Button_reset, self.reset_view_button)
        Kikuchi.setTabOrder(self.reset_view_button, self.Button_add_center)
        Kikuchi.setTabOrder(self.Button_add_center, self.centerLine)
        Kikuchi.setTabOrder(self.centerLine, self.ListBox_d_2)
        Kikuchi.setTabOrder(self.ListBox_d_2, self.Button_addband)
        Kikuchi.setTabOrder(self.Button_addband, self.Button_remove_band)
        Kikuchi.setTabOrder(self.Button_remove_band, self.tol_entry)
        Kikuchi.setTabOrder(self.tol_entry, self.plot_bands_button)
        Kikuchi.setTabOrder(self.plot_bands_button, self.Button_run)
        Kikuchi.setTabOrder(self.Button_run, self.ListBox_theo)
        Kikuchi.setTabOrder(self.ListBox_theo, self.Button_orientation)

    def retranslateUi(self, Kikuchi):
        _translate = QtCore.QCoreApplication.translate
        Kikuchi.setWindowTitle(_translate("Kikuchi", "Kikuchi"))
        self.groupBox_9.setTitle(_translate("Kikuchi", "Bands"))
        self.Button_addband.setText(_translate("Kikuchi", "Add bands"))
        self.Button_remove_band.setText(_translate("Kikuchi", "Remove band"))
        self.plot_bands_button.setText(_translate("Kikuchi", "Plot bands"))
        self.Button_add_center.setText(_translate("Kikuchi", "Add center"))
        self.label.setText(_translate("Kikuchi", "tol. angle"))
        self.Button_run.setText(_translate("Kikuchi", "Run"))
        self.Button_orientation.setText(_translate("Kikuchi", "Show bands"))
        self.groupBox.setTitle(_translate("Kikuchi", "Crystal Parameters"))
        self.Button_reset.setText(_translate("Kikuchi", "Reset points"))
        self.a_label.setText(_translate("Kikuchi", "a,b,c"))
        self.alpha_label.setText(_translate("Kikuchi", "<html><head/><body><p>α,β,γ  </p></body></html>"))
        self.indice_label.setText(_translate("Kikuchi", "Max Indices"))
        self.space_group_label.setText(_translate("Kikuchi", "Space group"))
        self.brightness_label.setText(_translate("Kikuchi", "Brightness"))
        self.Calib_label.setText(_translate("Kikuchi", "Calibrations"))
        self.reset_view_button.setText(_translate("Kikuchi", "Reset view"))
        self.menuSave.setTitle(_translate("Kikuchi", "Open"))
        self.menuStructure.setTitle(_translate("Kikuchi", "Structure"))
        self.menuRefine.setTitle(_translate("Kikuchi", "Tools"))
        self.actionSave_figure.setText(_translate("Kikuchi", "Open image"))
        self.actionCalculate_Schmid_factor.setText(_translate("Kikuchi", "calculate Schmid factor"))
        self.actionCalculate_spectrum.setText(_translate("Kikuchi", "Plot spectrum"))
        self.actionRefine_orientation.setText(_translate("Kikuchi", "Refine orientation"))
        self.actionGet_orientation_at_zero_tilt.setText(_translate("Kikuchi", "get orientation at zero-tilt"))
        self.actionTilt.setText(_translate("Kikuchi", "Tilt conditions"))



######################################################
#
# Interface is a script to solve interface planes from TEM BF/DF images at
# different tilts. It uses the approach developped by R.X. Xie from Tsinghua University
#
#######################################################


from __future__ import division
import numpy as np
from PyQt5 import QtGui, QtCore, QtWidgets
import sys
import os
from PIL import Image
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib import pyplot as plt
from . import interfaceUI, DrawInterfaceUI, DrawStretchedUI

######################
#
# click event control. Prevent to define a point when zooming
#
######################


def onpress(event):
    global press, move
    press = True


def onmove(event):
    global press, move
    if press:
        move = True


def onrelease(event):
    global press, move

    if press and not move:
        click(event)
    press = False
    move = False


def click(event):
    global gclick, s, minx, maxx, miny, maxy

    x = event.xdata
    y = event.ydata
    a.annotate(str(s), (x, y))
    a.plot(x, y, 'b+')
    a.axis('off')
    a.figure.canvas.draw()
    s = s + 1
    gclick = np.vstack((gclick, np.array([x, y])))

    return gclick, s


#########################
#
# Reset points
#
############################
def reset_last_point():
    global image_diff, gclick, s, minx, maxx, miny, maxy
    minx, maxx = a.get_xlim()
    miny, maxy = a.get_ylim()
    a.clear()
    img = Image.open(str(image_diff[0]))
    img = np.array(img)
    figure.suptitle(str(image_diff[0]))
    a.imshow(img, origin="upper")
    gclick = gclick[:-1, :]
    s = s - 1
    for i in range(1, gclick.shape[0]):
        a.annotate(str(i), (gclick[i, 0], gclick[i, 1]))
    a.plot(gclick[1:, 0], gclick[1:, 1], 'b+')

    a.axis([minx, maxx, miny, maxy])
    a.axis('off')
    a.figure.canvas.draw()

    return s, gclick


def reset_points():
    global image_diff, gclick, s, minx, maxx, miny, maxy, D_p
    minx, maxx = a.get_xlim()
    miny, maxy = a.get_ylim()
    a.clear()
    img = Image.open(str(image_diff[0]))
    img = np.array(img)
    figure.suptitle(str(image_diff[0]))
    a.imshow(img, origin="upper")
    a.axis([minx, maxx, miny, maxy])
    a.axis('off')
    a.figure.canvas.draw()
    s = 1
    gclick = np.zeros((1, 2))
    D_p = np.zeros(9)
    return s, gclick

######################
#
# reset view
#
#########################


def reset():
    global image_diff, gclick, s, minx, maxx, miny, maxy, D_p
    a.clear()
    img = Image.open(str(image_diff[0]))
    img = np.array(img)
    figure.suptitle(str(image_diff[0]))
    a.imshow(img, origin="upper")
    minx = 0
    maxx = width
    miny = height
    maxy = 0
    a.axis([minx, maxx, miny, maxy])
    D_p = np.zeros(9)
    for i in range(1, gclick.shape[0]):
        a.annotate(str(i), (gclick[i, 0], gclick[i, 1]))
    a.plot(gclick[1:, 0], gclick[1:, 1], 'b+')

    a.axis('off')
    a.figure.canvas.draw()
    ui.euler_Listbox.clear()

    return s, gclick


########################
#
# Add/Remove spot
#
#########################

def add_condition():
    global gclick, d, counter, x_calib
    x1 = gclick[1, 0]
    y1 = gclick[1, 1]
    x2 = gclick[2, 0]
    y2 = gclick[2, 1]
    if ui.direction_button.isChecked() is False:
        x3 = gclick[3, 0]
        y3 = gclick[3, 1]
        d = np.abs((y2 - y1) * x3 - (x2 - x1) * y3 + x2 * y1 - y2 * x1) / np.sqrt((y2 - y1)**2 + (x2 - x1)**2)
    else:
        d = np.sqrt((y2 - y1)**2 + (x2 - x1)**2)

    d12 = np.sqrt((x1 - x2)**2 + (y1 - y2)**2)
    if d12 == 0:
        bet = 0
    else:
        if x2 - x1 > 0:
            bet = 180 - np.arccos((y2 - y1) / (d12)) * 180 / np.pi
        else:
            bet = -180 + np.arccos((y2 - y1) / (d12)) * 180 / np.pi

    s3 = ui.tilt_y_entry.text()
    s4 = ui.tilt_x_entry.text()
    s5 = ui.tilt_z_entry.text()

    ii = 0
    while ii < len(x_micro):
        if x_micro[ii][0] == ui.micro_box.currentText():
            if ui.magnification_entry.text() == x_micro[ii][1]:
                s1 = d * float(x_micro[ii][2])
                s2 = bet + float(x_micro[ii][3])
        ii = ii + 1

    s = str(np.around(s1, decimals=3)) + ',' + str(np.around(s2, decimals=3)) + ',' + s3 + ',' + s4 + ',' + s5
    ui.conditions_Listbox.addItem(s)
    ss = ui.conditions_Listbox.count()
    item = ui.conditions_Listbox.item(ss - 1)
    item.setFlags(item.flags() | QtCore.Qt.ItemIsEditable)


def remove_condition():
    ui.conditions_Listbox.takeItem(ui.conditions_Listbox.currentRow())

######################################################
#
# Determine orientation from a set of at least 3 diff
#
####################################################


def unique_rows(a):
    a = np.ascontiguousarray(a)
    unique_a = np.unique(a.view([('', a.dtype)] * a.shape[1]))
    return unique_a.view(a.dtype).reshape((unique_a.shape[0], a.shape[1]))


def Rot(th, a, b, c):
    th = th * np.pi / 180
    aa = a / np.linalg.norm([a, b, c])
    bb = b / np.linalg.norm([a, b, c])
    cc = c / np.linalg.norm([a, b, c])
    c1 = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], float)
    c2 = np.array([[aa**2, aa * bb, aa * cc], [bb * aa, bb**2, bb * cc], [cc * aa,
                                                                          cc * bb, cc**2]], float)
    c3 = np.array([[0, -cc, bb], [cc, 0, -aa], [-bb, aa, 0]], float)
    R = np.cos(th) * c1 + (1 - np.cos(th)) * c2 + np.sin(th) * c3

    return R


def rotation(phi1, phi, phi2):
    phi1 = phi1 * np.pi / 180
    phi = phi * np.pi / 180
    phi2 = phi2 * np.pi / 180
    R = np.array([[np.cos(phi1) * np.cos(phi2) - np.cos(phi) * np.sin(phi1) * np.sin(phi2),
                   -np.cos(phi) * np.cos(phi2) * np.sin(phi1) - np.cos(phi1) *
                   np.sin(phi2), np.sin(phi) * np.sin(phi1)], [np.cos(phi2) * np.sin(phi1)
                                                               + np.cos(phi) * np.cos(phi1) * np.sin(phi2), np.cos(phi) * np.cos(phi1)
                                                               * np.cos(phi2) - np.sin(phi1) * np.sin(phi2), -np.cos(phi1) * np.sin(phi)],
                  [np.sin(phi) * np.sin(phi2), np.cos(phi2) * np.sin(phi), np.cos(phi)]], float)
    return R

#################
#
# Crystal
#
#################


def cryst():

    abc = ui.abc_entry.text().split(",")
    a = np.float64(abc[0])
    b = np.float64(abc[1])
    c = np.float64(abc[2])
    alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    alpha = np.float64(alphabetagamma[0]) * np.pi / 180
    beta = np.float64(alphabetagamma[1]) * np.pi / 180
    gamma = np.float64(alphabetagamma[2]) * np.pi / 180
    V = a * b * c * np.sqrt(1 - (np.cos(alpha)**2) - (np.cos(beta))**2 - (np.cos(gamma))**2 + 2 * np.cos(alpha) * np.cos(beta) * np.cos(gamma))
    D = np.array([[a, b * np.cos(gamma), c * np.cos(beta)], [0, b * np.sin(gamma), c * (np.cos(alpha) - np.cos(beta) * np.cos(gamma)) / np.sin(gamma)], [0, 0, V / (a * b * np.sin(gamma))]])
    M = np.transpose(D)
    return M


def tilt_axes():
    global s_a, s_b, s_z
    s_a, s_b, s_z = -1, -1, -1
    if ui.alpha_signBox.isChecked():
        s_a = 1
    if ui.beta_signBox.isChecked():
        s_b = 1
    if ui.theta_signBox.isChecked():
        s_b = 1
    return s_a, s_b, s_z

##############################################
#
# Get normal:
# Convention in holder coordinates: y is the primary tilt angle fixed in the holder frame. x is the secondary tilt fixed in the sample frame. z is the tilt axis fixed in the sample frame.
# B is the beam direction. The sample is tilted along y by -alpha, then tilted along x or z by -beta/-z angle. The projected trace is referenced with respect to the y direction
# The trace is solved by SVD using [BxTp]T=0
# The normal is solved using pseudo inverse [B,T]dn=[sign w sqrt(1-BT^2)]
# Residual errors are computed using the boostrap resampling
#
##################################################


def bootstrap_norm(data1, data2, nor, num_samples, alpha):
    n = len(data1)
    idx = np.random.randint(0, n, (num_samples, n))
    idx[:, -1] = n - 1
    samples1 = data1[idx]
    samples2 = data2[idx]
    sumvec = np.array([0, 0, 0])
    dev = []
    w = []
    nor = nor.reshape(3,)
    for i in range(0, num_samples):
        vec = np.dot(np.linalg.pinv(samples1[i]), samples2[i])
        vec = vec.reshape(3,)
        sumvec = vec.T + sumvec
        w = np.append(w, np.linalg.norm(vec))
        vec = vec / np.linalg.norm(vec)
        dev = np.append(dev, np.arccos(np.dot(nor, vec)) * 180 / np.pi)

    dev = np.nan_to_num(dev)
    stat = np.sort(dev, axis=None)
    w = np.nan_to_num(w)
    stat_w = np.sort(w, axis=None)

    return (sumvec / num_samples, stat[int(alpha * num_samples / 100)], np.abs(0.5 * (stat_w[int((100 - alpha) * num_samples / 100)] - stat_w[int(alpha * num_samples / 100)])))


def bootstrap_dir(data1, nor, num_samples, alpha):
    n = len(data1)
    idx = np.random.randint(0, n, (num_samples, n))
    idx[:, -1] = n - 1
    samples1 = data1[idx]
    sumvec = np.array([0, 0, 0])
    dev = []
    nor = nor.reshape(3,)
    for i in range(0, num_samples):
        u, s, v = np.linalg.svd(samples1[i].T)
        vec = u[:, 2]
        sumvec = vec.T + sumvec
        vec = vec / np.linalg.norm(vec)
        dev = np.append(dev, np.arccos(np.dot(nor, vec)) * 180 / np.pi)

    dev = np.nan_to_num(dev)
    stat = np.sort(dev, axis=None)

    return (sumvec / num_samples, stat[int(alpha * num_samples / 100)])


def get():
    if ui.direction_button.isChecked():
        get_direction()
    else:
        get_normal()


def deviation_thick(angle, S):
    r = np.sqrt(S[0]**2 + S[1]**2 + S[2]**2)
    t = np.arctan2(S[1], S[0]) * 180 / np.pi
    ph = np.arccos(S[2] / r) * 180 / np.pi
    Aa = []
    for g in np.linspace(-np.pi, np.pi, 10):
        for j in range(1, 10):
            Aa = np.append(Aa, np.dot(Rot(t, 0, 0, 1), np.dot(Rot(ph, 0, 1, 0), np.array([np.sin(g) * np.sin(angle / j * np.pi / 180), np.cos(g) * np.sin(angle / j * np.pi / 180), np.cos(angle / j * np.pi / 180)])))[2])

    m = np.abs(S[2] - np.amax(Aa))
    return m


def get_direction():
    ui.euler_Listbox.clear()
    s_a, s_b, s_z = tilt_axes()
    s = [str(x.text()) for x in ui.conditions_Listbox.selectedItems()]
    if len(s) < 3:
        ui.euler_Listbox.addItem("Number of inputs should be more than 3")
    else:
        tilt_a = []
        tilt_b = []
        tilt_z = []
        inclination = []
        d = []

        for i in range(0, len(s)):
            l = list(map(float, s[i].split(',')))
            tilt_a.append(l[2])
            tilt_b.append(l[3])
            tilt_z.append(l[4])
            inclination.append(l[1])
            d.append(l[0])

        inclination = np.array(inclination)
        tilt_a = np.array(tilt_a)
        tilt_b = np.array(tilt_b)
        tilt_z = np.array(tilt_z)
        d = np.array(d)

        L = np.zeros((3, np.shape(tilt_a)[0]))
        BxTp = np.zeros((3, np.shape(tilt_a)[0]))
        t_ang = np.float64(ui.image_angle_entry.text())

        for i in range(0, tilt_a.shape[0]):
            R = np.dot(Rot(s_z * tilt_z[i], 0, 0, 1), np.dot(Rot(s_b * tilt_b[i], 1, 0, 0), Rot(s_a * tilt_a[i], 0, 1, 0)))
            ny = inclination[i] * np.pi / 180
            t = np.array([-np.cos(ny), np.sin(ny), 0])
            BxTp[:, i] = np.dot(R, np.dot(Rot(t_ang, 0, 0, 1), t))
            BxTp[:, i] = BxTp[:, i] / np.linalg.norm(BxTp[:, i])
            L[:, i] = np.dot(R, np.dot(Rot(t_ang, 0, 0, 1), np.array([np.sin(ny), np.cos(ny), 0])))

        BxTp = np.hstack((BxTp, L)).T
        N = tilt_a.shape[0]
        shuffle = np.zeros((2**(N - 1) + 1, N))
        res = np.zeros(2**(N - 1))

        for i in range(0, 2**(N - 1)):
            bb = np.zeros((N))
            shuffle[i, :] = np.array(list(np.binary_repr(i, width=N)), dtype=int) * -1
            shuffle[shuffle == 0] = 1  # sgn(i)
            for j in range(0, N):
                bb[j] = d[j] * shuffle[i, j]

            b = np.hstack((np.zeros((tilt_a.shape[0])), bb))
            T = np.dot(np.linalg.pinv(BxTp), b.T)
            res[i] = np.linalg.norm(np.dot(BxTp, T) - b.T)

        mini = np.argmin(res)

        for j in range(0, N):
            bb[j] = d[j] * shuffle[mini, j]
        b = np.hstack((np.zeros((tilt_a.shape[0])), bb))
        T = np.dot(np.linalg.pinv(BxTp), b.T)
        T, dev_angle_T, dev_width = bootstrap_norm(BxTp, b.T, T / np.linalg.norm(T), 10000, 95)

        dT = T / np.linalg.norm(T)
        thick = np.abs(T[2])
        dev_t = deviation_thick(dev_angle_T, dT) * np.linalg.norm(T)
        dev_thick = (dev_t + dev_width) * np.abs(T[2])

        dev_thick = deviation_thick(dev_angle_T, dT) * (np.linalg.norm(T) + dev_width * np.abs(dT[2]))

        if ui.crystal_checkBox.isChecked():
            euler = ui.euler_entry.text().split(",")
            phi1 = np.float64(euler[0])
            phi = np.float64(euler[1])
            phi2 = np.float64(euler[2])
            M = cryst()
            T_cc = np.dot(np.linalg.inv(rotation(phi1, phi, phi2)), T / np.linalg.norm(T)).T
            T_cc = np.dot(M, T_cc.T)

        else:
            T_cc = T

        np.set_printoptions(suppress=True)
        ui.euler_Listbox.addItem('Direction')
        ui.euler_Listbox.addItem(str(np.around(T_cc[0], decimals=3)) + ',' + str(np.around(T_cc[1], decimals=3)) + ',' + str(np.around(T_cc[2], decimals=3)))
        ui.euler_Listbox.addItem('Error on trace direction (deg)')
        ui.euler_Listbox.addItem(str(np.around(dev_angle_T, decimals=3)))
        ui.euler_Listbox.addItem('Direction length')
        ui.euler_Listbox.addItem(str(np.around(np.linalg.norm(T), decimals=3)))
        ui.euler_Listbox.addItem('Error on direction width')
        ui.euler_Listbox.addItem(str(np.around(dev_width, decimals=3)))
        ui.euler_Listbox.addItem('Estimated thickness')
        ui.euler_Listbox.addItem(str(np.around(thick, decimals=3)) + '+/-' + str(np.around(dev_thick, decimals=3)))


def get_normal():

    ui.euler_Listbox.clear()
    s_a, s_b, s_z = tilt_axes()
    s = [str(x.text()) for x in ui.conditions_Listbox.selectedItems()]
    if len(s) < 3:
        ui.euler_Listbox.addItem("Number of spots should be between 3 and 5")
    else:
        tilt_a = []
        tilt_b = []
        tilt_z = []
        width_inclination = []

        for i in range(0, len(s)):
            l = list(map(float, s[i].split(',')))
            tilt_a.append(l[2])
            tilt_b.append(l[3])
            tilt_z.append(l[4])
            width_inclination.append(l[0:2])

        width_inclination = np.array(width_inclination)
        tilt_a = np.array(tilt_a)
        tilt_b = np.array(tilt_b)
        tilt_z = np.array(tilt_z)

        B = np.zeros((3, np.shape(tilt_a)[0]))
        BxTp = np.zeros((3, np.shape(tilt_a)[0]))

        t_ang = np.float64(ui.image_angle_entry.text())

        for i in range(0, tilt_a.shape[0]):
            R = np.dot(Rot(s_z * tilt_z[i], 0, 0, 1), np.dot(Rot(s_b * tilt_b[i], 1, 0, 0), Rot(s_a * tilt_a[i], 0, 1, 0)))
            B[:, i] = np.dot(R, np.array([0, 0, 1]))
            ny = width_inclination[i, 1] * np.pi / 180
            t = np.array([-np.cos(ny), np.sin(ny), 0])
            BxTp[:, i] = np.dot(R, np.dot(Rot(t_ang, 0, 0, 1), t))
            BxTp[:, i] = BxTp[:, i] / np.linalg.norm(BxTp[:, i])

        u, s, v = np.linalg.svd(BxTp)
        T = u[:, 2]
        T_mean, dev_angle_t = bootstrap_dir(BxTp.T, T / np.linalg.norm(T), 10000, 95)
        N = tilt_a.shape[0]
        shuffle = np.zeros((2**(N - 1) + 1, N))
        B = np.vstack((B.T, T))
        res = np.zeros(2**(N - 1))

        for i in range(0, 2**(N - 1)):
            b = np.zeros((1, N + 1))
            shuffle[i, :] = np.array(list(np.binary_repr(i, width=N)), dtype=int) * -1
            shuffle[shuffle == 0] = 1  # sgn(i)
            for j in range(0, N):
                b[:, j] = width_inclination[j, 0] * shuffle[i, j] * np.sqrt(1 - np.dot(B[j, :], T)**2)
            n = np.dot(np.linalg.pinv(B), b.T)
            res[i] = np.linalg.norm(np.dot(B, n) - b.T)

        mini = np.argmin(res)

        for j in range(0, N):
            b[:, j] = width_inclination[j, 0] * shuffle[mini, j] * np.sqrt(1 - np.dot(B[j, :], T)**2)
        n = np.dot(np.linalg.pinv(B), b.T)
        n, dev_angle_n, dev_width = bootstrap_norm(B, b.T, n / np.linalg.norm(n), 10000, 95)

        dn = n / np.linalg.norm(n)
        thick = np.linalg.norm(n) * np.sqrt(1 - dn[2]**2)
        dev_n = deviation_thick(dev_angle_n, dn) * np.linalg.norm(n)
        dev_thick = (dev_n + dev_width) * np.sqrt(1 - dn[2]**2)
        # in crystal coordinate
        if ui.crystal_checkBox.isChecked():
            euler = ui.euler_entry.text().split(",")
            phi1 = np.float64(euler[0])
            phi = np.float64(euler[1])
            phi2 = np.float64(euler[2])
            M = cryst()
            n_cc = np.dot(np.linalg.inv(rotation(phi1, phi, phi2)), n / np.linalg.norm(n)).T
            T_cc = np.dot(np.linalg.inv(rotation(phi1, phi, phi2)), T / np.linalg.norm(T)).T
            n_cc = np.dot(M, n_cc.T)
            T_cc = np.dot(M, T_cc.T)

        else:
            n_cc = dn
            T_cc = T

        np.set_printoptions(suppress=True)
        ui.euler_Listbox.addItem('Plane normal')
        ui.euler_Listbox.addItem(str(np.around(n_cc[0], decimals=3)) + ',' + str(np.around(n_cc[1], decimals=3)) + ',' + str(np.around(n_cc[2], decimals=3)))
        ui.euler_Listbox.addItem('Trace')
        ui.euler_Listbox.addItem(str(np.around(T_cc[0], decimals=3)) + ',' + str(np.around(T_cc[1], decimals=3)) + ',' + str(np.around(T_cc[2], decimals=3)))
        ui.euler_Listbox.addItem('Error on trace direction (deg)')
        ui.euler_Listbox.addItem(str(np.around(dev_angle_t, decimals=3)))
        ui.euler_Listbox.addItem('Error on plane normal (deg)')
        ui.euler_Listbox.addItem(str(np.around(dev_angle_n, decimals=3)))
        ui.euler_Listbox.addItem('Plane width')
        ui.euler_Listbox.addItem(str(np.around(np.linalg.norm(n), decimals=3)))
        ui.euler_Listbox.addItem('Error on plane width')
        ui.euler_Listbox.addItem(str(np.around(dev_width, decimals=3)))
        ui.euler_Listbox.addItem('Estimated thickness')
        ui.euler_Listbox.addItem(str(np.around(thick, decimals=3)) + '+/-' + str(np.around(dev_thick, decimals=3)))

##############################
#
# Draw planes or directions
#
################################


def plot_dir_planes():
    global D_p, minx, miny, maxx, maxy, image_diff
    a.clear()
    img = Image.open(str(image_diff[0]))
    img = np.array(img)
    figure.suptitle(str(image_diff[0]))
    a.imshow(img, origin="upper")
    a.axis([minx, maxx, miny, maxy])
    a.axis('off')

    for i in range(1, D_p.shape[0]):
        if D_p[i, 8] == 1:
            a.plot([D_p[i, 2], D_p[i, 2] + D_p[i, 0]], [D_p[i, 3], D_p[i, 3] - D_p[i, 1]], 'r-')
            if ui_draw.label_checkBox.isChecked():
                st = str(np.float64(D_p[i, 5])) + ',' + str(np.float64(D_p[i, 6])) + ',' + str(np.float64(D_p[i, 7]))
                a.annotate(st, (D_p[i, 2] + D_p[i, 0], D_p[i, 3] - D_p[i, 1]))
            a.axis('off')

        else:
            xw = D_p[i, 3] - D_p[i, 2] * D_p[i, 1] / np.sqrt(D_p[i, 1]**2 + D_p[i, 0]**2)
            yw = D_p[i, 4] - D_p[i, 2] * D_p[i, 0] / np.sqrt(D_p[i, 1]**2 + D_p[i, 0]**2)

            a.plot([D_p[i, 3] - 100 * D_p[i, 0], D_p[i, 3] + 100 * D_p[i, 0]], [D_p[i, 4] + 100 * D_p[i, 1], D_p[i, 4] - 100 * D_p[i, 1]], 'b-')
            a.plot([xw - 100 * D_p[i, 0], xw + 100 * D_p[i, 0]], [yw + 100 * D_p[i, 1], yw - 100 * D_p[i, 1]], 'g-')
            a.plot([D_p[i, 3], xw], [D_p[i, 4], yw], 'r-')
            a.axis('off')

            if ui_draw.label_checkBox.isChecked():
                st = str(np.float64(D_p[i, 5])) + ',' + str(np.float64(D_p[i, 6])) + ',' + str(np.float64(D_p[i, 7]))
                angp = np.arctan2(D_p[i, 1], D_p[i, 0]) * 180 / np.pi
                a.annotate(st, (xw, yw), rotation=angp, ha='center', va='center')

    a.figure.canvas.draw()


def undo_plot_dir_planes():
    global D_p
    D_p = D_p[:-1, :]
    plot_dir_planes()


def draw_planes_dir():
    global gclick, minx, miny, maxx, maxy, image_diff, D_p

    t = np.float64(ui_draw.thickness_entry.text())
    s_a, s_b, s_z = tilt_axes()
    t_ang = np.float64(ui.image_angle_entry.text())
    tilt_x = -s_b * np.float64(ui.tilt_x_entry.text())
    tilt_y = -s_a * np.float64(ui.tilt_y_entry.text())
    tilt_z = -s_z * np.float64(ui.tilt_z_entry.text())

    ii = 0
    while ii < len(x_micro):
        if x_micro[ii][0] == ui.micro_box.currentText():
            if ui.magnification_entry.text() == x_micro[ii][1]:
                mag_conv = float(x_micro[ii][2])

        ii = ii + 1

    nd = ui_draw.plane_entry.text().split(",")
    nd = np.array([np.float64(nd[0]), np.float64(nd[1]), np.float64(nd[2])])
    if ui.crystal_checkBox.isChecked():
        euler = ui.euler_entry.text().split(",")
        phi1 = np.float64(euler[0])
        phi = np.float64(euler[1])
        phi2 = np.float64(euler[2])
    else:
        phi1 = 0
        phi = 0
        phi2 = 0
    M = cryst()
    D = np.transpose(M)
    Dstar = np.transpose(np.linalg.inv(D))
    x = gclick[-1, 0]
    y = gclick[-1, 1]

    if ui_draw.surf_checkBox.isChecked():
        s = ui_draw.surface_entry.text().split(",")
        s = np.array([np.float64(s[0]), np.float64(s[1]), np.float64(s[2])])
        s = np.dot(Dstar, s)
        s = np.dot(rotation(phi1, phi, phi2), s)
        s = s / np.linalg.norm(s)
    else:
        s = np.array([0, 0, 1])

    s = np.dot(Rot(tilt_y, 0, 1, 0), np.dot(Rot(tilt_x, 1, 0, 0), np.dot(Rot(tilt_z, 0, 0, 1), np.dot(Rot(t_ang, 0, 0, 1), s))))

    if ui_draw.measure_checkBox.isChecked():
        x0 = gclick[-2, 0]
        x1 = gclick[-1, 0]
        y0 = gclick[-2, 1]
        y1 = gclick[-1, 1]
        vn = np.array([-y1 + y0, -x1 + x0, 0])
        leng = np.linalg.norm(vn)
        vn = vn / leng
        v_plan = np.dot(Rot(-tilt_z, 0, 0, 1), np.dot(Rot(-tilt_x, 1, 0, 0), np.dot(Rot(-tilt_y, 0, 1, 0), np.dot(Rot(t_ang, 0, 0, 1), vn))))
        v_plan = np.dot(np.linalg.inv(rotation(phi1, phi, phi2)), v_plan)
        nd = np.dot(Dstar, nd)
        intersec = np.cross(nd, v_plan)
        intersec_d = np.dot(np.linalg.inv(D), intersec)
        if ui_draw.hexa_Button.isChecked():
            intersec_d = np.array([(2 * intersec_d[0] - intersec_d[1]) / 3, (2 * intersec_d[1] - intersec_d[0]) / 3, intersec_d[2]])

        dire = intersec
        dire = np.dot(rotation(phi1, phi, phi2), dire)
        dire = dire / np.linalg.norm(dire)
        dire = np.dot(Rot(tilt_y, 0, 1, 0), np.dot(Rot(tilt_x, 1, 0, 0), np.dot(Rot(tilt_z, 0, 0, 1), np.dot(Rot(t_ang, 0, 0, 1), dire))))
        b = np.array([0, 0, 1])
        d_proj = (dire - np.dot(dire, b) * b) * t / np.dot(dire, s)
        dr = np.abs(leng * mag_conv * t / np.dot(dire, s) / np.linalg.norm(d_proj))

        te = 'Distance measured in the plane: ' + str(np.around(dr, decimals=2)) + '\n' + 'Direction in the plane: ' + str(np.around(100 * intersec_d[0], decimals=3)) + ',' + str(np.around(100 * intersec_d[1], decimals=3)) + ',' + str(np.around(100 * intersec_d[2], decimals=3)) + '\n' + 'Max proj length: ' + str(np.around(np.linalg.norm(d_proj), decimals=2)) + '\n' + 'Measured proj length: ' + str(np.around(leng * mag_conv, decimals=2))
        ui_draw.measure_label.setText(te)
        if leng * mag_conv > np.linalg.norm(d_proj):
            ui_draw.measure_label.setText(te + '\n' + str("Measure exceeds sample thickness"))

    else:
        ui_draw.measure_label.clear()
        sens = 1
        if ui_draw.inv_checkBox.isChecked():
            sens = -1

        if ui_draw.dir_checkBox.isChecked():
            if ui_draw.hexa_Button.isChecked():
                na = 2 * nd[0] + nd[1]
                n2a = 2 * nd[1] + nd[0]
                nd[0] = na
                nd[1] = n2a

            dire = np.dot(D, nd)
            dire = np.dot(rotation(phi1, phi, phi2), dire)
            dire = dire / np.linalg.norm(dire)
            dire = np.dot(Rot(tilt_y, 0, 1, 0), np.dot(Rot(tilt_x, 1, 0, 0), np.dot(Rot(tilt_z, 0, 0, 1), np.dot(Rot(t_ang, 0, 0, 1), dire))))
            b = np.array([0, 0, 1])
            d_proj = sens * (dire - np.dot(dire, b) * b) * t / np.dot(dire, s) / mag_conv
            D_p = np.vstack((D_p, np.array([d_proj[0], d_proj[1], x, y, nd[0], nd[1], nd[2], 0, 1])))

        else:
            plan = np.dot(Dstar, nd)
            plan = np.dot(rotation(phi1, phi, phi2), plan)
            plan = plan / np.linalg.norm(plan)
            plan = np.dot(Rot(tilt_y, 0, 1, 0), np.dot(Rot(tilt_x, 1, 0, 0), np.dot(Rot(tilt_z, 0, 0, 1), np.dot(Rot(t_ang, 0, 0, 1), plan))))

            T = sens * np.cross(plan, s)
            T = T / np.linalg.norm(T)
            w = plan[2] * t / np.sqrt((1 - np.dot(plan, s)**2) * (1 - T[2]**2)) / mag_conv
            D_p = np.vstack((D_p, np.array([T[0], T[1], w, x, y, nd[0], nd[1], nd[2], 0])))

            if ui_draw.proj_image_checkBox.isChecked():

                a_r = np.arctan2(T[0], T[1])
                a_g = np.abs(np.sqrt(1 - T[2]**2) / plan[2])
                ui_draw.measure_label.setText('Rotation, Stretch' + '\n' + str(np.around(a_r * 180 / np.pi, decimals=3)) + ',' + str(np.around(a_g, decimals=3)))
                a_stretched = figure_stretched.add_subplot(111)
                a_stretched.figure.clear()
                a_stretched = figure_stretched.add_subplot(111)
                img = Image.open(str(image_diff[0]))
                im_rotate = img.rotate(a_r * 180 / np.pi, expand=1)
                stretched_size = (int(im_rotate.size[0] * a_g), im_rotate.size[1])
                im_stretched = im_rotate.resize(stretched_size)
                a_stretched.imshow(im_stretched, origin='upper')
                if ui_draw.scale_checkBox.isChecked():
                    scale = np.float64(ui_draw.scale_entry.text())
                    a_stretched.plot([stretched_size[0] / 20, stretched_size[0] / 20 + scale / mag_conv], [stretched_size[1] / 20, stretched_size[1] / 20], 'w-', linewidth=1)
                    a_stretched.annotate(str(scale) + 'nm', (stretched_size[0] / 20 + scale / mag_conv + 5, stretched_size[1] / 20), color="white", backgroundcolor="black")

                a_stretched.axis('off')
                a_stretched.set_facecolor('m')
                a_stretched.figure.canvas.draw()
                Stretched.show()
        plot_dir_planes()


#########################
#
# Open image
#
############################


def open_image():
    global width, height, image_diff, s, gclick, minx, maxx, miny, maxy, press, move, D_p, a
    press = False
    move = False
    a = figure.add_subplot(111)
    a.figure.clear()
    a = figure.add_subplot(111)
    image_diff = QtWidgets.QFileDialog.getOpenFileName(Interface, "Open image file", "", "*.png *.jpg *.bmp *.tiff *.tif *.jpeg")
    img = Image.open(str(image_diff[0]))
    img = np.array(img)
    a.imshow(img, origin='upper')
    figure.suptitle(str(image_diff[0]))
    height, width = img.shape[0], img.shape[1]
    a.axis([0, width, height, 0])
    a.axis('off')
    a.figure.canvas.draw()
    s = 1
    gclick = np.zeros((1, 2))
    D_p = np.zeros(9)
    ui.euler_Listbox.clear()
    minx = 0
    maxx = width
    miny = height
    maxy = 0
    return s, gclick


##################################################
#
# Add matplotlib toolbar to zoom and pan
#
###################################################


class NavigationToolbar(NavigationToolbar):
    # only display the buttons we need
    toolitems = [t for t in NavigationToolbar.toolitems if
                 t[0] in ('Pan', 'Zoom')]

    def set_message(self, msg):
        pass

######################################################
# import crystal structures from un txt file Name,a,b,c,alpha,beta,gamma,space group
#
######################################################


def structure(item):
    global x0, e_entry
    ui.abc_entry.setText(str(item[1]) + ',' + str(item[2]) + ',' + str(item[3]))
    ui.alphabetagamma_entry.setText(str(item[4]) + ',' + str(item[5]) + ',' + str(item[6]))

##################
#
# Set tilting buttons
#
################


def single_tilt():
    ui.tilt_y_entry.setEnabled(True)
    ui.tilt_z_entry.setDisabled(True)
    ui.tilt_x_entry.setDisabled(True)
    ui.tilt_y_entry.setText('0')


def double_tilt():
    ui.tilt_z_entry.setDisabled(True)
    ui.tilt_x_entry.setEnabled(True)
    ui.tilt_y_entry.setEnabled(True)
    ui.tilt_y_entry.setText('0')
    ui.tilt_x_entry.setText('0')


def tilt_rot():
    ui.tilt_z_entry.setEnabled(True)
    ui.tilt_x_entry.setDisabled(True)
    ui.tilt_y_entry.setEnabled(True)
    ui.tilt_y_entry.setText('0')
    ui.tilt_z_entry.setText('0')


def import_data():
    ui.conditions_Listbox.clear()
    ui.euler_Listbox.clear()
    data_file = QtWidgets.QFileDialog.getOpenFileName(Interface, "Open a data file", "", "*.txt")
    data = open(data_file[0], 'r')
    x0 = []

    for line in data:
        line = line.strip()
        if not line:
            continue
        if line.startswith("#"):
            continue
        x0.append(list(map(str, line.split())))

    data.close()
    if len(x0[0][0].split(',')) == 3:
        ui.euler_entry.setText(str(x0[0][0]))
        x0 = x0[1:]
    for item in x0:
        ui.conditions_Listbox.addItem(item[0])


def export_data():
    s = [str(x.text()) for x in ui.conditions_Listbox.selectedItems()]
    res = [str(ui.euler_Listbox.item(i).text()) for i in range(ui.euler_Listbox.count())]
    name = QtWidgets.QFileDialog.getSaveFileName(Interface, 'Save File')
    fout = open(name[0], 'w')
    fout.write('# Interface data file \n')
    fout.write('# Euler angles \n')
    fout.write(str(ui.euler_entry.text()) + '\n')
    if ui.direction_button.isChecked():
        d = 'direction'
    else:
        d = 'plane'
    fout.write('# Data for a  ' + str(d) + '\n')
    for item in s:
        fout.write("%s\n" % item)

    fout.write('\n')
    fout.write('# Results \n')
    for item in res:
        fout.write("# %s\n" % item)

    fout.close()


def except_hook(cls, exception, traceback):
    sys.__excepthook__(cls, exception, traceback)


######################################################
#
# Launch
#
######################################################
try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

if __name__ == "__main__":

    app = QtWidgets.QApplication(sys.argv)
    sys.excepthook = except_hook
    QtWidgets.qApp.setApplicationName("Interface")
    Interface = QtWidgets.QMainWindow()
    ui = interfaceUI.Ui_Interface()
    ui.setupUi(Interface)
    figure = plt.figure()
    canvas = FigureCanvas(figure)
    ui.mplvl.addWidget(canvas)
    toolbar = NavigationToolbar(canvas, canvas)
    toolbar.setMinimumWidth(601)

    Draw = QtWidgets.QDialog()
    ui_draw = DrawInterfaceUI.Ui_Draw_plane_directions()
    ui_draw.setupUi(Draw)
    ui.actionDirections_planes.triggered.connect(Draw.show)
    ui_draw.buttonBox.rejected.connect(Draw.close)
    ui_draw.buttonBox.accepted.connect(draw_planes_dir)

    ui.actionImport.triggered.connect(import_data)
    ui.actionExport.triggered.connect(export_data)

    Stretched = QtWidgets.QDialog()
    ui_stretched = DrawStretchedUI.Ui_Draw_Stretched()
    ui_stretched.setupUi(Stretched)
    figure_stretched = plt.figure()
    figure_stretched.patch.set_facecolor('black')
    canvas_stretched = FigureCanvas(figure_stretched)
    ui_stretched.mplvl.addWidget(canvas_stretched)
    toolbar_stretched = NavigationToolbar(canvas_stretched, canvas_stretched)
    toolbar_stretched.setMinimumWidth(101)
    toolbar_stretched.setStyleSheet("background-color:White;")

    single_tilt()
    ui.single_button.setChecked(True)
    ui.tilt_x_entry.setText('0')
    ui.tilt_z_entry.setText('0')
    ui.single_button.toggled.connect(single_tilt)
    ui.double_button.toggled.connect(double_tilt)
    ui.tilt_rot_button.toggled.connect(tilt_rot)
    ui.magnification_entry.setText('1')
    ui.image_angle_entry.setText('0')
    s = 1
    gclick = np.zeros((1, 2))
    D_p = np.zeros(9)

    file_struct = open(os.path.join(os.path.dirname(__file__), 'structure.txt'), "r")

    x0 = []

    for line in file_struct:
        x0.append(list(map(str, line.split())))

    i = 0
    file_struct.close()

    for item in x0:
        entry = ui.menuStructure.addAction(item[0])
        entry.triggered.connect(lambda checked, item=item: structure(item))
        i = i + 1

#########################
#
# Get magnification and rotation angle between holder and image from the microscope.txt file
#
############################

    f_micro = open(os.path.join(os.path.dirname(__file__), 'microscope.txt'), "r")

    x_micro = []

    for line in f_micro:
        x_micro.append(list(map(str, line.split())))

    x_micro = np.array(x_micro)
    x_micro_unique, ind_x = np.unique(x_micro[:, 0], return_index=True)

    for i in ind_x[::-1]:
        ui.micro_box.addItem(x_micro[i][0])

    f_micro.close()
# Ctrl+w shortcut to remove clicked pole

    shortcut = QtWidgets.QShortcut(QtGui.QKeySequence("Ctrl+w"), Interface)
    shortcut.activated.connect(reset_last_point)

    ui.actionSave_figure.triggered.connect(open_image)

    figure.canvas.mpl_connect('button_press_event', onpress)
    figure.canvas.mpl_connect('button_release_event', onrelease)
    figure.canvas.mpl_connect('motion_notify_event', onmove)
    press = False
    move = False

    ui.Button_reset.clicked.connect(reset_points)
    ui.reset_view_button.clicked.connect(reset)

    ui.add_condition_button.clicked.connect(add_condition)
    ui.remove_condition_button.clicked.connect(remove_condition)
    ui.normal_button.clicked.connect(get)
    shortcut_plot = QtWidgets .QShortcut(QtGui.QKeySequence("Ctrl+w"), Draw)
    shortcut_plot.activated.connect(undo_plot_dir_planes)

    Interface.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'angleUI.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Angle(object):
    def setupUi(self, Angle):
        Angle.setObjectName("Angle")
        Angle.resize(285, 234)
        self.layoutWidget = QtWidgets.QWidget(Angle)
        self.layoutWidget.setGeometry(QtCore.QRect(10, 10, 266, 216))
        self.layoutWidget.setObjectName("layoutWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.layoutWidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.buttonBox = QtWidgets.QDialogButtonBox(self.layoutWidget)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 7, 0, 1, 2)
        self.angle_label = QtWidgets.QLabel(self.layoutWidget)
        self.angle_label.setText("")
        self.angle_label.setObjectName("angle_label")
        self.gridLayout.addWidget(self.angle_label, 6, 0, 1, 2)
        self.n2_label = QtWidgets.QLabel(self.layoutWidget)
        self.n2_label.setObjectName("n2_label")
        self.gridLayout.addWidget(self.n2_label, 3, 0, 1, 1)
        self.n1_label = QtWidgets.QLabel(self.layoutWidget)
        self.n1_label.setObjectName("n1_label")
        self.gridLayout.addWidget(self.n1_label, 0, 0, 1, 1)
        self.n2_entry = QtWidgets.QLineEdit(self.layoutWidget)
        self.n2_entry.setObjectName("n2_entry")
        self.gridLayout.addWidget(self.n2_entry, 4, 0, 2, 1)
        self.n1_entry = QtWidgets.QLineEdit(self.layoutWidget)
        self.n1_entry.setObjectName("n1_entry")
        self.gridLayout.addWidget(self.n1_entry, 1, 0, 2, 1)
        self.uvw1_button = QtWidgets.QCheckBox(self.layoutWidget)
        self.uvw1_button.setObjectName("uvw1_button")
        self.gridLayout.addWidget(self.uvw1_button, 1, 1, 2, 1)
        self.uvw2_button = QtWidgets.QCheckBox(self.layoutWidget)
        self.uvw2_button.setObjectName("uvw2_button")
        self.gridLayout.addWidget(self.uvw2_button, 4, 1, 2, 1)

        self.retranslateUi(Angle)
        QtCore.QMetaObject.connectSlotsByName(Angle)
        Angle.setTabOrder(self.n1_entry, self.n2_entry)
        Angle.setTabOrder(self.n2_entry, self.buttonBox)

    def retranslateUi(self, Angle):
        _translate = QtCore.QCoreApplication.translate
        Angle.setWindowTitle(_translate("Angle", "Angle"))
        self.n2_label.setText(_translate("Angle", "n2/d2"))
        self.n1_label.setText(_translate("Angle", "n1/d1"))
        self.uvw1_button.setText(_translate("Angle", "uvw"))
        self.uvw2_button.setText(_translate("Angle", "uvw"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'listUI.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_List(object):
    def setupUi(self, List):
        List.setObjectName("List")
        List.resize(452, 557)
        self.gridLayout = QtWidgets.QGridLayout(List)
        self.gridLayout.setObjectName("gridLayout")
        self.ZA_button = QtWidgets.QCheckBox(List)
        self.ZA_button.setObjectName("ZA_button")
        self.gridLayout.addWidget(self.ZA_button, 4, 5, 1, 1)
        self.list_button = QtWidgets.QPushButton(List)
        self.list_button.setObjectName("list_button")
        self.gridLayout.addWidget(self.list_button, 1, 0, 1, 6)
        self.beta_button = QtWidgets.QRadioButton(List)
        self.beta_button.setObjectName("beta_button")
        self.gridLayout.addWidget(self.beta_button, 4, 3, 1, 1)
        self.z_button = QtWidgets.QRadioButton(List)
        self.z_button.setObjectName("z_button")
        self.gridLayout.addWidget(self.z_button, 4, 2, 1, 1)
        self.list_table = QtWidgets.QTableWidget(List)
        self.list_table.setObjectName("list_table")
        self.list_table.setColumnCount(0)
        self.list_table.setRowCount(0)
        self.gridLayout.addWidget(self.list_table, 0, 0, 1, 6)
        self.tilt_label = QtWidgets.QLabel(List)
        self.tilt_label.setObjectName("tilt_label")
        self.gridLayout.addWidget(self.tilt_label, 4, 0, 1, 1)
        self.alpha_button = QtWidgets.QPushButton(List)
        self.alpha_button.setObjectName("alpha_button")
        self.gridLayout.addWidget(self.alpha_button, 10, 0, 1, 6)
        self.tilt_entry = QtWidgets.QLineEdit(List)
        self.tilt_entry.setObjectName("tilt_entry")
        self.gridLayout.addWidget(self.tilt_entry, 4, 1, 1, 1)
        self.plot_button = QtWidgets.QPushButton(List)
        self.plot_button.setObjectName("plot_button")
        self.gridLayout.addWidget(self.plot_button, 2, 0, 1, 3)
        self.plane_checkBox = QtWidgets.QCheckBox(List)
        self.plane_checkBox.setObjectName("plane_checkBox")
        self.gridLayout.addWidget(self.plane_checkBox, 2, 3, 1, 1)
        self.cone_checkBox = QtWidgets.QCheckBox(List)
        self.cone_checkBox.setObjectName("cone_checkBox")
        self.gridLayout.addWidget(self.cone_checkBox, 2, 5, 1, 1)

        self.retranslateUi(List)
        QtCore.QMetaObject.connectSlotsByName(List)
        List.setTabOrder(self.list_table, self.list_button)
        List.setTabOrder(self.list_button, self.plot_button)
        List.setTabOrder(self.plot_button, self.plane_checkBox)
        List.setTabOrder(self.plane_checkBox, self.cone_checkBox)
        List.setTabOrder(self.cone_checkBox, self.tilt_entry)
        List.setTabOrder(self.tilt_entry, self.z_button)
        List.setTabOrder(self.z_button, self.beta_button)
        List.setTabOrder(self.beta_button, self.ZA_button)
        List.setTabOrder(self.ZA_button, self.alpha_button)

    def retranslateUi(self, List):
        _translate = QtCore.QCoreApplication.translate
        List.setWindowTitle(_translate("List", "List"))
        self.ZA_button.setText(_translate("List", "Zone Axis"))
        self.list_button.setText(_translate("List", "Update list"))
        self.beta_button.setText(_translate("List", "β tilt"))
        self.z_button.setText(_translate("List", "θ tilt"))
        self.tilt_label.setText(_translate("List", "tilt"))
        self.alpha_button.setText(_translate("List", "Compute tilt"))
        self.plot_button.setText(_translate("List", "Add/Remove selected"))
        self.plane_checkBox.setText(_translate("List", "plane"))
        self.cone_checkBox.setText(_translate("List", "cone"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'interfaceUI.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets
import os
fn = os.path.join(os.path.dirname(__file__), "interface-icon.png")

class Ui_Interface(object):
    def setupUi(self, Interface):
        Interface.setObjectName("Interface")
        Interface.resize(1200, 755)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(fn), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Interface.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(Interface)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.mplvl = QtWidgets.QGridLayout()
        self.mplvl.setObjectName("mplvl")
        self.gridLayout_7.addLayout(self.mplvl, 0, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox.setMaximumSize(QtCore.QSize(400, 16777215))
        self.groupBox.setTitle("")
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.Button_reset = QtWidgets.QPushButton(self.groupBox)
        self.Button_reset.setObjectName("Button_reset")
        self.gridLayout.addWidget(self.Button_reset, 37, 0, 1, 3)
        self.alphabetagamma_label = QtWidgets.QLabel(self.groupBox)
        self.alphabetagamma_label.setObjectName("alphabetagamma_label")
        self.gridLayout.addWidget(self.alphabetagamma_label, 2, 0, 1, 1)
        self.abc_label = QtWidgets.QLabel(self.groupBox)
        self.abc_label.setObjectName("abc_label")
        self.gridLayout.addWidget(self.abc_label, 0, 0, 1, 1)
        self.magnification_entry = QtWidgets.QLineEdit(self.groupBox)
        self.magnification_entry.setObjectName("magnification_entry")
        self.gridLayout.addWidget(self.magnification_entry, 24, 2, 1, 2)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 26, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 8, 0, 1, 1)
        self.remove_condition_button = QtWidgets.QPushButton(self.groupBox)
        self.remove_condition_button.setObjectName("remove_condition_button")
        self.gridLayout.addWidget(self.remove_condition_button, 27, 3, 1, 1)
        self.conditions_Listbox = QtWidgets.QListWidget(self.groupBox)
        self.conditions_Listbox.setMaximumSize(QtCore.QSize(16777215, 100))
        self.conditions_Listbox.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.conditions_Listbox.setObjectName("conditions_Listbox")
        self.gridLayout.addWidget(self.conditions_Listbox, 28, 0, 1, 5)
        self.z_label = QtWidgets.QLabel(self.groupBox)
        self.z_label.setObjectName("z_label")
        self.gridLayout.addWidget(self.z_label, 23, 0, 1, 1)
        self.tilt_y_entry = QtWidgets.QLineEdit(self.groupBox)
        self.tilt_y_entry.setObjectName("tilt_y_entry")
        self.gridLayout.addWidget(self.tilt_y_entry, 19, 2, 1, 2)
        self.add_condition_button = QtWidgets.QPushButton(self.groupBox)
        self.add_condition_button.setObjectName("add_condition_button")
        self.gridLayout.addWidget(self.add_condition_button, 27, 0, 1, 3)
        self.tilt_z_entry = QtWidgets.QLineEdit(self.groupBox)
        self.tilt_z_entry.setObjectName("tilt_z_entry")
        self.gridLayout.addWidget(self.tilt_z_entry, 23, 2, 1, 2)
        self.double_button = QtWidgets.QRadioButton(self.groupBox)
        self.double_button.setObjectName("double_button")
        self.gridLayout.addWidget(self.double_button, 15, 3, 1, 1)
        self.image_angle_entry = QtWidgets.QLineEdit(self.groupBox)
        self.image_angle_entry.setObjectName("image_angle_entry")
        self.gridLayout.addWidget(self.image_angle_entry, 12, 3, 1, 2)
        self.abc_entry = QtWidgets.QLineEdit(self.groupBox)
        self.abc_entry.setObjectName("abc_entry")
        self.gridLayout.addWidget(self.abc_entry, 0, 2, 1, 3)
        self.euler_label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.euler_label.setFont(font)
        self.euler_label.setObjectName("euler_label")
        self.gridLayout.addWidget(self.euler_label, 3, 0, 1, 1)
        self.image_angle_label = QtWidgets.QLabel(self.groupBox)
        self.image_angle_label.setObjectName("image_angle_label")
        self.gridLayout.addWidget(self.image_angle_label, 12, 0, 1, 3)
        self.alphabetagamma_entry = QtWidgets.QLineEdit(self.groupBox)
        self.alphabetagamma_entry.setObjectName("alphabetagamma_entry")
        self.gridLayout.addWidget(self.alphabetagamma_entry, 2, 2, 1, 3)
        self.micro_box = QtWidgets.QComboBox(self.groupBox)
        self.micro_box.setObjectName("micro_box")
        self.gridLayout.addWidget(self.micro_box, 8, 2, 1, 3)
        self.pxtonm_label = QtWidgets.QLabel(self.groupBox)
        self.pxtonm_label.setObjectName("pxtonm_label")
        self.gridLayout.addWidget(self.pxtonm_label, 24, 0, 1, 1)
        self.reset_view_button = QtWidgets.QPushButton(self.groupBox)
        self.reset_view_button.setObjectName("reset_view_button")
        self.gridLayout.addWidget(self.reset_view_button, 37, 3, 1, 1)
        self.tilt_rot_button = QtWidgets.QRadioButton(self.groupBox)
        self.tilt_rot_button.setObjectName("tilt_rot_button")
        self.gridLayout.addWidget(self.tilt_rot_button, 15, 4, 1, 1)
        self.beta_label_2 = QtWidgets.QLabel(self.groupBox)
        self.beta_label_2.setObjectName("beta_label_2")
        self.gridLayout.addWidget(self.beta_label_2, 21, 0, 1, 1)
        self.crystal_checkBox = QtWidgets.QCheckBox(self.groupBox)
        self.crystal_checkBox.setObjectName("crystal_checkBox")
        self.gridLayout.addWidget(self.crystal_checkBox, 7, 0, 1, 5)
        self.euler_entry = QtWidgets.QLineEdit(self.groupBox)
        self.euler_entry.setObjectName("euler_entry")
        self.gridLayout.addWidget(self.euler_entry, 3, 2, 1, 3)
        self.normal_button = QtWidgets.QPushButton(self.groupBox)
        self.normal_button.setObjectName("normal_button")
        self.gridLayout.addWidget(self.normal_button, 39, 0, 1, 5)
        self.euler_Listbox = QtWidgets.QListWidget(self.groupBox)
        self.euler_Listbox.setMaximumSize(QtCore.QSize(16777215, 100))
        self.euler_Listbox.setObjectName("euler_Listbox")
        self.gridLayout.addWidget(self.euler_Listbox, 40, 0, 1, 5)
        self.theta_signBox = QtWidgets.QCheckBox(self.groupBox)
        self.theta_signBox.setText("")
        self.theta_signBox.setObjectName("theta_signBox")
        self.gridLayout.addWidget(self.theta_signBox, 23, 4, 1, 1)
        self.single_button = QtWidgets.QRadioButton(self.groupBox)
        self.single_button.setObjectName("single_button")
        self.gridLayout.addWidget(self.single_button, 15, 0, 1, 1)
        self.alpha_signBox = QtWidgets.QCheckBox(self.groupBox)
        self.alpha_signBox.setObjectName("alpha_signBox")
        self.gridLayout.addWidget(self.alpha_signBox, 19, 4, 1, 1)
        self.beta_signBox = QtWidgets.QCheckBox(self.groupBox)
        self.beta_signBox.setText("")
        self.beta_signBox.setObjectName("beta_signBox")
        self.gridLayout.addWidget(self.beta_signBox, 21, 4, 1, 1)
        self.tilt_x_entry = QtWidgets.QLineEdit(self.groupBox)
        self.tilt_x_entry.setObjectName("tilt_x_entry")
        self.gridLayout.addWidget(self.tilt_x_entry, 21, 2, 1, 2)
        self.alpha_label_2 = QtWidgets.QLabel(self.groupBox)
        self.alpha_label_2.setObjectName("alpha_label_2")
        self.gridLayout.addWidget(self.alpha_label_2, 19, 0, 1, 1)
        self.direction_button = QtWidgets.QCheckBox(self.groupBox)
        self.direction_button.setObjectName("direction_button")
        self.gridLayout.addWidget(self.direction_button, 26, 2, 1, 2)
        self.gridLayout_7.addWidget(self.groupBox, 0, 1, 1, 1)
        Interface.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(Interface)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1200, 22))
        self.menubar.setObjectName("menubar")
        self.menuSave = QtWidgets.QMenu(self.menubar)
        self.menuSave.setObjectName("menuSave")
        self.menuStructure = QtWidgets.QMenu(self.menubar)
        self.menuStructure.setObjectName("menuStructure")
        self.menuDraw = QtWidgets.QMenu(self.menubar)
        self.menuDraw.setObjectName("menuDraw")
        self.menuData = QtWidgets.QMenu(self.menubar)
        self.menuData.setObjectName("menuData")
        Interface.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(Interface)
        self.statusbar.setObjectName("statusbar")
        Interface.setStatusBar(self.statusbar)
        self.actionSave_figure = QtWidgets.QAction(Interface)
        self.actionSave_figure.setObjectName("actionSave_figure")
        self.actionCalculate_Schmid_factor = QtWidgets.QAction(Interface)
        self.actionCalculate_Schmid_factor.setObjectName("actionCalculate_Schmid_factor")
        self.actionCalculate_spectrum = QtWidgets.QAction(Interface)
        self.actionCalculate_spectrum.setObjectName("actionCalculate_spectrum")
        self.actionDirections_planes = QtWidgets.QAction(Interface)
        self.actionDirections_planes.setObjectName("actionDirections_planes")
        self.actionImport = QtWidgets.QAction(Interface)
        self.actionImport.setObjectName("actionImport")
        self.actionExport = QtWidgets.QAction(Interface)
        self.actionExport.setObjectName("actionExport")
        self.menuSave.addAction(self.actionSave_figure)
        self.menuDraw.addAction(self.actionDirections_planes)
        self.menuData.addAction(self.actionImport)
        self.menuData.addAction(self.actionExport)
        self.menubar.addAction(self.menuSave.menuAction())
        self.menubar.addAction(self.menuStructure.menuAction())
        self.menubar.addAction(self.menuDraw.menuAction())
        self.menubar.addAction(self.menuData.menuAction())

        self.retranslateUi(Interface)
        QtCore.QMetaObject.connectSlotsByName(Interface)
        Interface.setTabOrder(self.abc_entry, self.alphabetagamma_entry)
        Interface.setTabOrder(self.alphabetagamma_entry, self.euler_entry)
        Interface.setTabOrder(self.euler_entry, self.crystal_checkBox)
        Interface.setTabOrder(self.crystal_checkBox, self.micro_box)
        Interface.setTabOrder(self.micro_box, self.image_angle_entry)
        Interface.setTabOrder(self.image_angle_entry, self.single_button)
        Interface.setTabOrder(self.single_button, self.double_button)
        Interface.setTabOrder(self.double_button, self.tilt_rot_button)
        Interface.setTabOrder(self.tilt_rot_button, self.tilt_y_entry)
        Interface.setTabOrder(self.tilt_y_entry, self.alpha_signBox)
        Interface.setTabOrder(self.alpha_signBox, self.tilt_x_entry)
        Interface.setTabOrder(self.tilt_x_entry, self.beta_signBox)
        Interface.setTabOrder(self.beta_signBox, self.tilt_z_entry)
        Interface.setTabOrder(self.tilt_z_entry, self.theta_signBox)
        Interface.setTabOrder(self.theta_signBox, self.magnification_entry)
        Interface.setTabOrder(self.magnification_entry, self.direction_button)
        Interface.setTabOrder(self.direction_button, self.add_condition_button)
        Interface.setTabOrder(self.add_condition_button, self.remove_condition_button)
        Interface.setTabOrder(self.remove_condition_button, self.conditions_Listbox)
        Interface.setTabOrder(self.conditions_Listbox, self.Button_reset)
        Interface.setTabOrder(self.Button_reset, self.reset_view_button)
        Interface.setTabOrder(self.reset_view_button, self.normal_button)
        Interface.setTabOrder(self.normal_button, self.euler_Listbox)

    def retranslateUi(self, Interface):
        _translate = QtCore.QCoreApplication.translate
        Interface.setWindowTitle(_translate("Interface", "interface determination"))
        self.Button_reset.setText(_translate("Interface", "Reset points"))
        self.alphabetagamma_label.setText(_translate("Interface", "<html><head/><body><p>α, β, γ</p></body></html>"))
        self.abc_label.setText(_translate("Interface", "a,b,c"))
        self.label.setText(_translate("Interface", "<html><head/><body><p><span style=\" font-weight:600;\">Measure</span></p></body></html>"))
        self.label_2.setText(_translate("Interface", "<html><head/><body><p><span style=\" font-weight:600;\">Microscope</span></p></body></html>"))
        self.remove_condition_button.setText(_translate("Interface", "Remove"))
        self.z_label.setText(_translate("Interface", "θ  (°)"))
        self.add_condition_button.setText(_translate("Interface", "Add data"))
        self.double_button.setText(_translate("Interface", "double (α,β)"))
        self.euler_label.setText(_translate("Interface", "<html><head/><body><p><span style=\" font-weight:400;\">Euler angles</span></p></body></html>"))
        self.image_angle_label.setText(_translate("Interface", "<html><head/><body><p>Image-α tilt angle</p></body></html>"))
        self.pxtonm_label.setText(_translate("Interface", "Magnification"))
        self.reset_view_button.setText(_translate("Interface", "Reset view"))
        self.tilt_rot_button.setText(_translate("Interface", "tilt rot (α,θ)"))
        self.beta_label_2.setText(_translate("Interface", "<html><head/><body><p>β (°)</p></body></html>"))
        self.crystal_checkBox.setText(_translate("Interface", "Use lattice coordinates"))
        self.normal_button.setText(_translate("Interface", "Get normal/direction"))
        self.single_button.setText(_translate("Interface", "single (α)"))
        self.alpha_signBox.setText(_translate("Interface", "AC"))
        self.alpha_label_2.setText(_translate("Interface", "<html><head/><body><p>α (°)</p></body></html>"))
        self.direction_button.setText(_translate("Interface", "direction"))
        self.menuSave.setTitle(_translate("Interface", "Open"))
        self.menuStructure.setTitle(_translate("Interface", "Structure"))
        self.menuDraw.setTitle(_translate("Interface", "Draw"))
        self.menuData.setTitle(_translate("Interface", "Data"))
        self.actionSave_figure.setText(_translate("Interface", "Open image"))
        self.actionCalculate_Schmid_factor.setText(_translate("Interface", "calculate Schmid factor"))
        self.actionCalculate_spectrum.setText(_translate("Interface", "Plot spectrum"))
        self.actionDirections_planes.setText(_translate("Interface", "directions/planes"))
        self.actionImport.setText(_translate("Interface", "Import"))
        self.actionExport.setText(_translate("Interface", "Export"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'DrawInterfaceUI.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Draw_plane_directions(object):
    def setupUi(self, Draw_plane_directions):
        Draw_plane_directions.setObjectName("Draw_plane_directions")
        Draw_plane_directions.resize(399, 281)
        self.gridLayout_2 = QtWidgets.QGridLayout(Draw_plane_directions)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.plane_label = QtWidgets.QLabel(Draw_plane_directions)
        self.plane_label.setObjectName("plane_label")
        self.gridLayout.addWidget(self.plane_label, 2, 0, 2, 1)
        self.surf_checkBox = QtWidgets.QCheckBox(Draw_plane_directions)
        self.surf_checkBox.setObjectName("surf_checkBox")
        self.gridLayout.addWidget(self.surf_checkBox, 1, 3, 1, 1)
        self.label_checkBox = QtWidgets.QCheckBox(Draw_plane_directions)
        self.label_checkBox.setObjectName("label_checkBox")
        self.gridLayout.addWidget(self.label_checkBox, 4, 0, 1, 1)
        self.surface_label = QtWidgets.QLabel(Draw_plane_directions)
        self.surface_label.setObjectName("surface_label")
        self.gridLayout.addWidget(self.surface_label, 1, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(Draw_plane_directions)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 7, 0, 1, 4)
        self.thickness_label = QtWidgets.QLabel(Draw_plane_directions)
        self.thickness_label.setObjectName("thickness_label")
        self.gridLayout.addWidget(self.thickness_label, 0, 0, 1, 1)
        self.hexa_Button = QtWidgets.QCheckBox(Draw_plane_directions)
        self.hexa_Button.setObjectName("hexa_Button")
        self.gridLayout.addWidget(self.hexa_Button, 4, 3, 1, 1)
        self.measure_checkBox = QtWidgets.QCheckBox(Draw_plane_directions)
        self.measure_checkBox.setObjectName("measure_checkBox")
        self.gridLayout.addWidget(self.measure_checkBox, 4, 1, 1, 1)
        self.measure_label = QtWidgets.QLabel(Draw_plane_directions)
        self.measure_label.setText("")
        self.measure_label.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.measure_label.setObjectName("measure_label")
        self.gridLayout.addWidget(self.measure_label, 6, 0, 1, 4)
        self.thickness_entry = QtWidgets.QLineEdit(Draw_plane_directions)
        self.thickness_entry.setObjectName("thickness_entry")
        self.gridLayout.addWidget(self.thickness_entry, 0, 1, 1, 2)
        self.surface_entry = QtWidgets.QLineEdit(Draw_plane_directions)
        self.surface_entry.setObjectName("surface_entry")
        self.gridLayout.addWidget(self.surface_entry, 1, 1, 1, 2)
        self.proj_image_checkBox = QtWidgets.QCheckBox(Draw_plane_directions)
        self.proj_image_checkBox.setObjectName("proj_image_checkBox")
        self.gridLayout.addWidget(self.proj_image_checkBox, 5, 0, 1, 1)
        self.scale_checkBox = QtWidgets.QCheckBox(Draw_plane_directions)
        self.scale_checkBox.setObjectName("scale_checkBox")
        self.gridLayout.addWidget(self.scale_checkBox, 5, 3, 1, 1)
        self.scale_entry = QtWidgets.QLineEdit(Draw_plane_directions)
        self.scale_entry.setObjectName("scale_entry")
        self.gridLayout.addWidget(self.scale_entry, 5, 2, 1, 1)
        self.plane_entry = QtWidgets.QLineEdit(Draw_plane_directions)
        self.plane_entry.setObjectName("plane_entry")
        self.gridLayout.addWidget(self.plane_entry, 2, 1, 2, 2)
        self.dir_checkBox = QtWidgets.QCheckBox(Draw_plane_directions)
        self.dir_checkBox.setObjectName("dir_checkBox")
        self.gridLayout.addWidget(self.dir_checkBox, 2, 3, 1, 1)
        self.inv_checkBox = QtWidgets.QCheckBox(Draw_plane_directions)
        self.inv_checkBox.setObjectName("inv_checkBox")
        self.gridLayout.addWidget(self.inv_checkBox, 3, 3, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)

        self.retranslateUi(Draw_plane_directions)
        QtCore.QMetaObject.connectSlotsByName(Draw_plane_directions)
        Draw_plane_directions.setTabOrder(self.thickness_entry, self.surface_entry)
        Draw_plane_directions.setTabOrder(self.surface_entry, self.surf_checkBox)
        Draw_plane_directions.setTabOrder(self.surf_checkBox, self.plane_entry)
        Draw_plane_directions.setTabOrder(self.plane_entry, self.dir_checkBox)
        Draw_plane_directions.setTabOrder(self.dir_checkBox, self.inv_checkBox)
        Draw_plane_directions.setTabOrder(self.inv_checkBox, self.label_checkBox)
        Draw_plane_directions.setTabOrder(self.label_checkBox, self.measure_checkBox)
        Draw_plane_directions.setTabOrder(self.measure_checkBox, self.hexa_Button)
        Draw_plane_directions.setTabOrder(self.hexa_Button, self.proj_image_checkBox)
        Draw_plane_directions.setTabOrder(self.proj_image_checkBox, self.scale_entry)
        Draw_plane_directions.setTabOrder(self.scale_entry, self.scale_checkBox)

    def retranslateUi(self, Draw_plane_directions):
        _translate = QtCore.QCoreApplication.translate
        Draw_plane_directions.setWindowTitle(_translate("Draw_plane_directions", "Draw planes-directions"))
        self.plane_label.setText(_translate("Draw_plane_directions", "Plane/Direction"))
        self.surf_checkBox.setText(_translate("Draw_plane_directions", "surf."))
        self.label_checkBox.setText(_translate("Draw_plane_directions", "label"))
        self.surface_label.setText(_translate("Draw_plane_directions", "Surface norm."))
        self.thickness_label.setText(_translate("Draw_plane_directions", "Thickness (nm)"))
        self.hexa_Button.setText(_translate("Draw_plane_directions", "hexa"))
        self.measure_checkBox.setText(_translate("Draw_plane_directions", "measure"))
        self.proj_image_checkBox.setText(_translate("Draw_plane_directions", "proj. image"))
        self.scale_checkBox.setText(_translate("Draw_plane_directions", "scale (nm)"))
        self.dir_checkBox.setText(_translate("Draw_plane_directions", "dir."))
        self.inv_checkBox.setText(_translate("Draw_plane_directions", "inv"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'DrawStretchedUI.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Draw_Stretched(object):
    def setupUi(self, Draw_Stretched):
        Draw_Stretched.setObjectName("Draw_Stretched")
        Draw_Stretched.resize(772, 711)
        self.gridLayout = QtWidgets.QGridLayout(Draw_Stretched)
        self.gridLayout.setObjectName("gridLayout")
        self.mplvl = QtWidgets.QGridLayout()
        self.mplvl.setObjectName("mplvl")
        self.gridLayout.addLayout(self.mplvl, 0, 0, 1, 1)

        self.retranslateUi(Draw_Stretched)
        QtCore.QMetaObject.connectSlotsByName(Draw_Stretched)

    def retranslateUi(self, Draw_Stretched):
        _translate = QtCore.QCoreApplication.translate
        Draw_Stretched.setWindowTitle(_translate("Draw_Stretched", "Draw stretched image"))



from setuptools import find_packages, setup

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

setup(
    name = 'pycotem',
    packages=find_packages(exclude=("doc","test")),
    version = '2.6.0',  
    description = 'A python package for working with crystal orientations in transmission electron microscopy',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author = 'f. mompiou',
    author_email = 'frederic.mompiou@cemes.fr',
    license='GPL-3.0',
    url = 'https://mompiou.github.io/pycotem/',
    download_url = 'https://github.com/mompiou/pycotem',
    keywords = ['scientific', 'crystallography', 'electron microscopy'],
    classifiers = ['Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3',],
    include_package_data=True,
)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tiltUI.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Tilt(object):
    def setupUi(self, Tilt):
        Tilt.setObjectName("Tilt")
        Tilt.resize(396, 117)
        self.gridLayout = QtWidgets.QGridLayout(Tilt)
        self.gridLayout.setObjectName("gridLayout")
        self.theta_signBox = QtWidgets.QCheckBox(Tilt)
        self.theta_signBox.setText("")
        self.theta_signBox.setObjectName("theta_signBox")
        self.gridLayout.addWidget(self.theta_signBox, 1, 8, 1, 1)
        self.beta_label = QtWidgets.QLabel(Tilt)
        self.beta_label.setObjectName("beta_label")
        self.gridLayout.addWidget(self.beta_label, 1, 3, 1, 1)
        self.alpha_signBox = QtWidgets.QCheckBox(Tilt)
        self.alpha_signBox.setText("")
        self.alpha_signBox.setObjectName("alpha_signBox")
        self.gridLayout.addWidget(self.alpha_signBox, 1, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(Tilt)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 5)
        self.tilt_z_entry = QtWidgets.QLineEdit(Tilt)
        self.tilt_z_entry.setObjectName("tilt_z_entry")
        self.gridLayout.addWidget(self.tilt_z_entry, 1, 7, 1, 1)
        self.tilt_a_entry = QtWidgets.QLineEdit(Tilt)
        self.tilt_a_entry.setObjectName("tilt_a_entry")
        self.gridLayout.addWidget(self.tilt_a_entry, 1, 1, 1, 1)
        self.beta_signBox = QtWidgets.QCheckBox(Tilt)
        self.beta_signBox.setText("")
        self.beta_signBox.setObjectName("beta_signBox")
        self.gridLayout.addWidget(self.beta_signBox, 1, 5, 1, 1)
        self.alpha_label_2 = QtWidgets.QLabel(Tilt)
        self.alpha_label_2.setObjectName("alpha_label_2")
        self.gridLayout.addWidget(self.alpha_label_2, 1, 0, 1, 1)
        self.tilt_b_entry = QtWidgets.QLineEdit(Tilt)
        self.tilt_b_entry.setObjectName("tilt_b_entry")
        self.gridLayout.addWidget(self.tilt_b_entry, 1, 4, 1, 1)
        self.z_label = QtWidgets.QLabel(Tilt)
        self.z_label.setObjectName("z_label")
        self.gridLayout.addWidget(self.z_label, 1, 6, 1, 1)
        self.t_ang_entry = QtWidgets.QLineEdit(Tilt)
        self.t_ang_entry.setObjectName("t_ang_entry")
        self.gridLayout.addWidget(self.t_ang_entry, 2, 5, 1, 4)

        self.retranslateUi(Tilt)
        QtCore.QMetaObject.connectSlotsByName(Tilt)

    def retranslateUi(self, Tilt):
        _translate = QtCore.QCoreApplication.translate
        Tilt.setWindowTitle(_translate("Tilt", "Tilt conditions"))
        self.beta_label.setText(_translate("Tilt", "β (°)"))
        self.label_3.setText(_translate("Tilt", "Diffraction-α tilt angle"))
        self.alpha_label_2.setText(_translate("Tilt", "α (°)"))
        self.z_label.setText(_translate("Tilt", "θ (°)"))



##################
#
# Kikuchi is a tool to orient crystals from kikuchi lines in TEM
#
##################


from __future__ import division
import numpy as np
from PyQt5 import QtGui, QtCore, QtWidgets
import sys
import os
from PIL import Image
from PIL import ImageEnhance
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib import pyplot as plt
from . import kikuchiUI
from . import refineUI
from . import tiltUI

######################
#
# click event control. Prevent to define a point when zooming
#
######################


def onpress(event):
    global press, move
    press = True


def onmove(event):
    global press, move
    if press:
        move = True


def onrelease(event):
    global press, move

    if press and not move:
        click(event)
    press = False
    move = False

######################################
#
# Store clicked positions on the image
#
######################################


def click(event):
    global gclick, s, minx, maxx, miny, maxy
    x = event.xdata
    y = event.ydata
    a.annotate(str(s), (x, y))
    a.plot(x, y, 'b+')
    a.axis('off')
    a.figure.canvas.draw()
    s = s + 1
    gclick = np.vstack((gclick, np.array([x, y])))

    return gclick, s


########################
#
# Erase clicked points , and center
#
#####################

def reset_points():
    global image_diff, gclick, minx, maxx, miny, maxy, s
    minx, maxx = a.get_xlim()
    miny, maxy = a.get_ylim()
    brightness()
    a.axis([minx, maxx, miny, maxy])
    a.axis('off')
    a.figure.canvas.draw()
    s = 1
    gclick = np.zeros((1, 2))
    ui.ListBox_theo.clear()
    return s, gclick


def reset():
    global image_diff, gclick, s, minx, maxx, miny, maxy
    a.clear()
    img = Image.open(str(image_diff[0]))
    img = np.array(img)
    figure.suptitle(str(image_diff[0]))
    a.imshow(img, origin="upper")
    minx = 0
    maxx = width
    miny = height
    maxy = 0
    a.axis([minx, maxx, miny, maxy])
    a.axis('off')
    a.figure.canvas.draw()
    gclick = np.zeros((1, 2))
    ui.ListBox_theo.clear()
    brightness()
    s = 1
    return s, gclick


def remove_center():
    ui.centerLine.clear()

################################################################
#
# Determine interplanar distances from the crystal parameters
# and the space group
# Return a list of h,k,l indices close to the measured distance d  in Dist
# Return also the angles between hkl directions Ang
#
###################################################################


def listb():
    global Dist, Ang, G, Dstar
    abc = ui.abc_entry.text().split(",")
    a = np.float64(abc[0])
    b = np.float64(abc[1])
    c = np.float64(abc[2])
    alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    alp = np.float64(alphabetagamma[0])
    bet = np.float64(alphabetagamma[1])
    gam = np.float64(alphabetagamma[2])
    e = np.int32(ui.indice_entry.text())
    alp = alp * np.pi / 180
    bet = bet * np.pi / 180
    gam = gam * np.pi / 180
    Dist = np.zeros(((2 * e + 1)**3 - 1, 5))
    G = np.array([[a**2, a * b * np.cos(gam), a * c * np.cos(bet)], [a * b * np.cos(gam), b**2, b * c * np.cos(alp)], [a * c * np.cos(bet), b * c * np.cos(alp), c**2]])
    V = a * b * c * np.sqrt(1 - (np.cos(alp)**2) - (np.cos(bet))**2 - (np.cos(gam))**2 + 2 * np.cos(alp) * np.cos(bet) * np.cos(gam))
    D = np.array([[a, b * np.cos(gam), c * np.cos(bet)], [0, b * np.sin(gam), c * (np.cos(alp) - np.cos(bet) * np.cos(gam)) / np.sin(gam)], [0, 0, V / (a * b * np.sin(gam))]])
    Dstar = np.transpose(np.linalg.inv(D))
    w = 0
    for i in range(-e, e + 1):
        for j in range(-e, e + 1):
            for k in range(-e, e + 1):
                if (i, j, k) != (0, 0, 0):
                    di = 1 / (np.sqrt(np.dot(np.array([i, j, k]), np.dot(np.linalg.inv(G), np.array([i, j, k])))))
                    I = extinction(ui.space_group_Box.currentText(), i, j, k)
                    if I != 0:
                        Dist[w, :] = np.array([np.around(di, decimals=3), i, j, k, I])
                        w = w + 1

    Dist = Dist[Dist[:, 0].argsort()]
    Dist = Dist[~np.all(Dist == 0, axis=1)]
    Dist = Dist[::-1]

    lenD = np.shape(Dist)[0]
    Ang = np.zeros((lenD, lenD))

    for i in range(lenD):
        T = np.zeros(lenD)
        for j in range(lenD):
            c1c = np.dot(Dstar, Dist[i, 1:4])
            c2c = np.dot(Dstar, Dist[j, 1:4])
            T[j] = np.around(np.arccos(np.dot(c1c, c2c) / (np.linalg.norm(c1c) * np.linalg.norm(c2c))) * 180 / np.pi, decimals=3)

        Ang[i, :] = T

    return Dist, Ang

##################################
# Return interplanar distance
##################################


def distance(i, j, k):
    abc = ui.abc_entry.text().split(",")
    a = np.float64(abc[0])
    b = np.float64(abc[1])
    c = np.float64(abc[2])
    alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    alp = np.float64(alphabetagamma[0])
    bet = np.float64(alphabetagamma[1])
    gam = np.float64(alphabetagamma[2])
    alp = alp * np.pi / 180
    bet = bet * np.pi / 180
    gam = gam * np.pi / 180
    G = np.array([[a**2, a * b * np.cos(gam), a * c * np.cos(bet)], [a * b * np.cos(gam), b**2, b * c * np.cos(alp)], [a * c * np.cos(bet), b * c * np.cos(alp), c**2]])
    di = 1 / (np.sqrt(np.dot(np.array([i, j, k]), np.dot(np.linalg.inv(G), np.array([i, j, k])))))

    return di


#############################################
#
# Select the band and store it in B2 array. From the points determine m, b coefficient for the line equation y=mx+d0. d is the calibrated distance
#
#########################################################################

def addband():
    global gclick, d, counter, x_calib, height, B2

    Pt1 = gclick[1]
    Pt1[1] = -Pt1[1] + height
    Pt2 = gclick[2]
    Pt2[1] = -Pt2[1] + height
    Pt3 = gclick[3]
    Pt3[1] = -Pt3[1] + height
    m = (Pt2[1] - Pt1[1]) / (Pt2[0] - Pt1[0])
    b = (Pt1[1] * Pt2[0] - Pt2[1] * Pt1[0]) / (Pt2[0] - Pt1[0])
    d0 = (m * Pt3[0] - Pt3[1] + b) / np.sqrt(1 + m**2)
    B2 = np.vstack((B2, [m, b, d0]))
    ui.ListBox_d_2.addItem(str(np.shape(B2)[0] - 2) + ', dp (px):' + str(np.around(np.abs(d0), decimals=3)))
    reset_points()

    return B2

#############################################
#
# Remove a band from the list
#
############################################


def removeband():
    global B2

    g = ui.ListBox_d_2.currentRow()
    if g > -1:
        B2 = np.delete(B2, (ui.ListBox_d_2.currentRow() + 1), axis=0)
        ui.ListBox_d_2.takeItem(ui.ListBox_d_2.currentRow())

    return B2

#############################################
#
# Add the transmitted beam position
#
############################################


def addcenter():
    global Ct
    Ct = gclick[1]
    ui.centerLine.setText(str(np.around(Ct[0], decimals=1)) + ',' + str(np.around(Ct[1], decimals=1)))
    reset_points()
    return Ct

#############################################
#
# Open an image.
#
############################################


def open_image():
    global width, height, s, gclick, mean_ih, image_diff, B2, press, move, minx, maxx, miny, maxy, a
    press = False
    move = False
    a.clear()
    image_diff = QtWidgets.QFileDialog.getOpenFileName(Index, "Open image file", "", "*.png *.jpg *.bmp *.tiff *.tif *.jpeg")
    img = Image.open(str(image_diff[0]))
    ih = img.convert('L').histogram()
    mean_ih = sum(ih[g] * g / sum(ih) for g in range(len(ih)))
    a.imshow(img, origin='upper')
    figure.suptitle(str(image_diff[0]))
    width, height = img.size
    a.axis([0, width, height, 0])
    a.axis('off')
    a.figure.canvas.draw()
    ui.ListBox_d_2.clear()
    s = 1
    gclick = np.zeros((1, 2))
    B2 = np.zeros((1, 3))
    ui.ListBox_theo.clear()
    ui.ListBox_d_2.clear()
    ui.brightness_slider.setValue(100)
    minx = 0
    maxx = width
    miny = height
    maxy = 0

    return s, gclick


##############################################
#
# Rotation matrix for Euler angle and rotation along axis
#
##############################################


def rotation(phi1, phi, phi2):
    phi1 = phi1 * np.pi / 180
    phi = phi * np.pi / 180
    phi2 = phi2 * np.pi / 180
    R = np.array([[np.cos(phi1) * np.cos(phi2) - np.cos(phi) * np.sin(phi1) * np.sin(phi2),
                   -np.cos(phi) * np.cos(phi2) * np.sin(phi1) - np.cos(phi1) *
                   np.sin(phi2), np.sin(phi) * np.sin(phi1)], [np.cos(phi2) * np.sin(phi1)
                                                               + np.cos(phi) * np.cos(phi1) * np.sin(phi2), np.cos(phi) * np.cos(phi1)
                                                               * np.cos(phi2) - np.sin(phi1) * np.sin(phi2), -np.cos(phi1) * np.sin(phi)],
                  [np.sin(phi) * np.sin(phi2), np.cos(phi2) * np.sin(phi), np.cos(phi)]], float)
    return R


def Rot(th, a, b, c):
    th = th * np.pi / 180
    aa = a / np.linalg.norm([a, b, c])
    bb = b / np.linalg.norm([a, b, c])
    cc = c / np.linalg.norm([a, b, c])
    c1 = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], float)
    c2 = np.array([[aa**2, aa * bb, aa * cc], [bb * aa, bb**2, bb * cc], [cc * aa,
                                                                          cc * bb, cc**2]], float)
    c3 = np.array([[0, -cc, bb], [cc, 0, -aa], [-bb, aa, 0]], float)
    R = np.cos(th) * c1 + (1 - np.cos(th)) * c2 + np.sin(th) * c3

    return R

##############################################
#
# Determine  the orientation: get the beam direction from two intersecting bands. A third band is necessary to discriminate the correct indices of the bands
# Inputs are stored in the B2=[m,p,d,h,k,l] matrix from the bands
#
##############################################

##############################################
# Get the third coordinate a g vector from the camera length L
##############################################


def three_coor(vect, L):

    c = np.linalg.norm(vect)**2 / L
    vector = [vect[0], vect[1], -c]
    return vector

##############################################################
#
# Test functions for 2 to 6 bands. Search for conditions where the theoretical angles stored in Tab of length P0 are in agreement with the measured ones stored in tab. The min misorientation is 0.5 degrees. It can be increased up to 3 degrees by 0.5 degree step until match. Return a list of  g vectors
#
#################################################################


def testangle2(tab):
    global P0, Tab, eps
    liste_possibles = []
    while liste_possibles == []:
        for ii in P0:

            T1 = np.where(np.abs(Tab[ii, :] - tab[0, 1]) <= eps)

            for jj in T1[0]:
                T = [list(list2[ii].astype(np.int32)),
                     list(list2[jj].astype(np.int32))]
                liste_possibles.append(T)

        if eps < 3:
            eps += 0.5
            ui.ListBox_theo.addItem('Still running...')
        else:
            ui.ListBox_theo.addItem('No match')
            break

    return liste_possibles


def testangle3(tab):
    global P0, Tab, eps
    liste_possibles = []
    while liste_possibles == []:
        for ii in P0:

            T1 = np.where(np.abs(Tab[ii, :] - tab[0, 1]) <= eps)
            T2 = np.where(np.abs(Tab[ii, :] - tab[0, 2]) <= eps)
            for jj in T1[0]:
                for kk in T2[0]:
                    if np.abs(Tab[kk, jj] - tab[1, 2]) <= eps:
                        T = [list(list2[ii].astype(np.int32)),
                             list(list2[jj].astype(np.int32)),
                             list(list2[kk].astype(np.int32))]
                        liste_possibles.append(T)

        if eps < 3:
            eps += 0.5
            ui.ListBox_theo.addItem('Still running...')
        else:
            ui.ListBox_theo.addItem('No match')
            break

    return liste_possibles


def testangle4(tab):
    global P0, Tab, eps
    liste_possibles = []
    while liste_possibles == []:
        for ii in P0:
            T1 = np.where(np.abs(Tab[ii, :] - tab[0, 1]) <= eps)
            T2 = np.where(np.abs(Tab[ii, :] - tab[0, 2]) <= eps)
            T3 = np.where(np.abs(Tab[ii, :] - tab[0, 3]) <= eps)
            for jj in T1[0]:
                for kk in T2[0]:
                    for pp in T3[0]:
                        if np.abs(Tab[kk, jj] - tab[1, 2]) <= eps and np.abs(Tab[pp, jj] - tab[1, 3]) <= eps \
                                and np.abs(Tab[kk, pp] - tab[2, 3]) <= eps:
                            T = [list(list2[ii].astype(np.int32)),
                                 list(list2[jj].astype(np.int32)),
                                 list(list2[kk].astype(np.int32)),
                                 list(list2[pp].astype(np.int32))]
                            liste_possibles.append(T)

        if eps < 3:
            eps += 0.5
            ui.ListBox_theo.addItem('Still running...')
        else:
            ui.ListBox_theo.addItem('No match')
            break

    return liste_possibles


def testangle5(tab):
    global P0, Tab, eps
    liste_possibles = []
    while liste_possibles == []:
        for ii in P0:
            T1 = np.where(np.abs(Tab[ii, :] - tab[0, 1]) <= eps)
            T2 = np.where(np.abs(Tab[ii, :] - tab[0, 2]) <= eps)
            T3 = np.where(np.abs(Tab[ii, :] - tab[0, 3]) <= eps)
            T4 = np.where(np.abs(Tab[ii, :] - tab[0, 4]) <= eps)
            for jj in T1[0]:
                for kk in T2[0]:
                    for pp in T3[0]:
                        for hh in T4[0]:
                            if np.abs(Tab[kk, jj] - tab[1, 2]) <= eps and np.abs(Tab[pp, jj] - tab[1, 3]) <= eps \
                                    and np.abs(Tab[kk, pp] - tab[2, 3]) <= eps and np.abs(Tab[hh, jj] - tab[1, 4]) <= eps \
                                    and np.abs(Tab[hh, kk] - tab[2, 4]) <= eps and np.abs(Tab[hh, pp] - tab[3, 4]) <= eps:
                                T = [list(list2[ii].astype(np.int32)),
                                     list(list2[jj].astype(np.int32)),
                                     list(list2[kk].astype(np.int32)),
                                     list(list2[pp].astype(np.int32)),
                                     list(list2[hh].astype(np.int32))]
                                liste_possibles.append(T)

        if eps < 3:
            ui.ListBox_theo.addItem('Still running...')
            eps += 0.5
        else:
            ui.ListBox_theo.addItem('No match')
            break

    return liste_possibles


def testangle6(tab):
    global P0, Tab, eps
    liste_possibles = []
    while liste_possibles == []:
        for ii in P0:
            T1 = np.where(np.abs(Tab[ii, :] - tab[0, 1]) <= eps)
            T2 = np.where(np.abs(Tab[ii, :] - tab[0, 2]) <= eps)
            T3 = np.where(np.abs(Tab[ii, :] - tab[0, 3]) <= eps)
            T4 = np.where(np.abs(Tab[ii, :] - tab[0, 4]) <= eps)
            T5 = np.where(np.abs(Tab[ii, :] - tab[0, 5]) <= eps)
            for jj in T1[0]:
                for kk in T2[0]:
                    for pp in T3[0]:
                        for hh in T4[0]:
                            for ll in T5[0]:
                                if np.abs(Tab[kk, jj] - tab[1, 2]) <= eps and np.abs(Tab[pp, jj] - tab[1, 3]) <= eps \
                                        and np.abs(Tab[kk, pp] - tab[2, 3]) <= eps and np.abs(Tab[hh, jj] - tab[1, 4]) <= eps \
                                        and np.abs(Tab[hh, kk] - tab[2, 4]) <= eps and np.abs(Tab[hh, pp] - tab[3, 4]) <= eps \
                                        and np.abs(Tab[ll, jj] - tab[1, 5]) <= eps and np.abs(Tab[kk, ll] - tab[2, 5]) <= eps \
                                        and np.abs(Tab[ll, pp] - tab[3, 5]) <= eps and np.abs(Tab[hh, ll] - tab[4, 5]) <= eps:
                                    T = [list(list2[ii].astype(np.int32)),
                                         list(list2[jj].astype(np.int32)),
                                         list(list2[kk].astype(np.int32)),
                                         list(list2[pp].astype(np.int32)),
                                         list(list2[hh].astype(np.int32)),
                                         list(list2[ll].astype(np.int32))]
                                    liste_possibles.append(T)

        if eps < 3:
            ui.ListBox_theo.addItem('Still running...')
            eps += 0.5
        else:
            ui.ListBox_theo.addItem('No match')
            break

    return liste_possibles

##############################################
#
# Test the uniqueness of a set of solution. Return a vector d with the interplanar distance of the row vector of A
#
##############################################


def Uniqueness(A):
    l = np.shape(A)[0]
    abc = ui.abc_entry.text().split(",")
    a = np.float64(abc[0])
    b = np.float64(abc[1])
    c = np.float64(abc[2])
    alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    alp = np.float64(alphabetagamma[0])
    bet = np.float64(alphabetagamma[1])
    gam = np.float64(alphabetagamma[2])
    alp = alp * np.pi / 180
    bet = bet * np.pi / 180
    gam = gam * np.pi / 180
    G = np.array([[a**2, a * b * np.cos(gam), a * c * np.cos(bet)], [a * b * np.cos(gam), b**2, b * c * np.cos(alp)], [a * c * np.cos(bet), b * c * np.cos(alp), c**2]])
    d = np.zeros((1, l))

    for t in range(0, l):
        d[:, t] = 1 / (np.sqrt(np.dot(A[t, :], np.dot(np.linalg.inv(G), A[t, :]))))

    return d


def tilt_axes():
    global s_a, s_b, s_z
    s_a, s_b, s_z = -1, -1, -1
    if ui_Tilt.alpha_signBox.isChecked():
        s_a = 1
    if ui_Tilt.beta_signBox.isChecked():
        s_b = 1
    if ui_Tilt.theta_signBox.isChecked():
        s_z = 1
    return s_a, s_b, s_z


def rot_tilt_angle():
    tilt_axes()
    tilt_a = np.float64(ui_Tilt.tilt_a_entry.text())
    tilt_b = np.float64(ui_Tilt.tilt_b_entry.text())
    tilt_z = np.float64(ui_Tilt.tilt_z_entry.text())
    t_ang = np.float64(ui_Tilt.t_ang_entry.text())
    R = np.dot(Rot(s_z * tilt_z, 0, 0, 1), np.dot(Rot(s_b * tilt_b, 1, 0, 0), np.dot(Rot(s_a * tilt_a, 0, 1, 0), Rot(t_ang, 0, 0, 1))))
    return R

##############################################################################
#
#  Show results function
# 1. Get the electron wavelength lambd from acceleration voltage
# 2. Camera length L in Px  L=r(px)*d/lamba, r measured distance in pixel, d the reticular spacing in Angstrom, lambda wavelength, r.d being the calibration constant in the calibration.txt file
# 3. For all the bands in B2, calculate the normal vectors to the bands v0 and their normalized 3D value v and the value of p (in the equation y=mx+p)
# 4. Calculate the angle between the normal vectors and store in Tab
# 5. Test the angles with the testangle function and store the possibilities in the K array and keep only the unique solutions
# 6. Compute orientation from Mackenzie approach
# 7. Display the solutions in the results box

#################################################################################


def show_result():
    global Ct, B2, Dist, P0, eps, Tab, list2, tab, K, v0, pOB, L, v, euler, lambd, calib, euler_tilt
    ui.ListBox_theo.clear()
    if ui_Refine.L_entry.text() == '':
        ui_Refine.L_entry.setText(str(x_calib[ui.Calib_box.currentIndex()][4]))
    if ui_Refine.V_entry.text() == '':
        ui_Refine.V_entry.setText(str(x_calib[ui.Calib_box.currentIndex()][1]))
    calib = np.float64(ui_Refine.L_entry.text())
    Vt = np.float64(ui_Refine.V_entry.text()) * 1e3

    h = 6.62607004e-34
    m0 = 9.10938356e-31
    e = 1.60217662e-19
    c = 299792458
    lambd = h / np.sqrt(2 * m0 * e * Vt) * 1 / np.sqrt(1 + e * Vt / (2 * m0 * c**2)) * 1e10

    L = calib / lambd
    list2, Tab = listb()
    eps = np.float64(ui.tol_entry.text())
    P0 = range(len(list2))
    Num = np.shape(B2)[0]
    v = np.zeros((Num - 1, 3))
    v0 = np.zeros((Num - 1, 2))
    dp = np.zeros((Num - 1, 1))
    for z in range(0, Num - 1):
        v0[z, :] = np.array([(Ct[0] + B2[z + 1, 0] * (height - Ct[1]) - B2[z + 1, 1] * B2[z + 1, 0]) / (1 + B2[z + 1, 0]**2) - Ct[0], height - (B2[z + 1, 0] * (Ct[0] + B2[z + 1, 0] * (height - Ct[1]) - B2[z + 1, 1] * B2[z + 1, 0]) / (1 + B2[z + 1, 0]**2) + B2[z + 1, 1]) - Ct[1]])
        g = (1 + 0.5 * np.sign(v0[z, 1]) * B2[z + 1, 2] / np.linalg.norm(v0[z, :]))
        v0[z, :] = -v0[z, :] * g
        v[z, :] = three_coor(v0[z, :], L)
        dp[z] = calib / 2 / np.linalg.norm(v[z, :] - three_coor(v0[z, :] / g, L))
        v[z, :] = v[z, :] / np.linalg.norm(v[z, :])

    v[:, 1] = -v[:, 1]
    tab = np.zeros((np.shape(v)[0], np.shape(v)[0]))
    for l in range(0, np.shape(v)[0]):
        for f in range(0, np.shape(v)[0]):
            tab[l, f] = np.arccos(np.dot(v[l, :], v[f, :])) * 180 / np.pi

    n = np.shape(tab)[0]
    if eps < 3:
        if n == 2:
            K = testangle2(tab)
        elif n == 3:
            K = testangle3(tab)
        elif n == 4:
            K = testangle4(tab)
        elif n == 5:
            K = testangle5(tab)
        elif n == 6:
            K = testangle6(tab)
        else:
            ui.ListBox_theo.addItem('Number of bands should be more than 1 \n and less than 7')
            return
    else:
        ui.ListBox_theo.addItem('tol. should be below 3 degrees')
        return

    K = np.asarray(K)
    U = np.zeros((np.shape(K)[0], Num - 1))
    if K.shape[0] > 0:
        for t in range(0, np.shape(K)[0]):
            U[t, :] = Uniqueness(K[t, :, 1:4])

        K = K[np.unique(U, return_index=True, axis=0)[1], :, :]
        euler = np.zeros((K.shape[0], 5))
        for sol in range(0, K.shape[0]):
            g_c = K[sol, :, 1:4]
            g_c = np.dot(Dstar, g_c.T)
            n_c = np.linalg.norm(g_c.T, axis=1)
            g_c = (g_c / n_c).T
            g_sample = v
            if np.abs(np.linalg.det(np.dot(g_c.T, g_sample))) < 1e-7:
                g_cc = np.cross(g_c[0, :], g_c[1, :])
                g_sc = np.cross(g_sample[0, :], g_sample[1, :])
                g_sc = g_sc / np.linalg.norm(g_sc)
                g_cc = g_cc / np.linalg.norm(g_cc)
                g_c = np.vstack((g_c, g_cc))
                g_sample = np.vstack((g_sample, g_sc))
            if np.linalg.det(np.dot(g_c.T, g_sample)) < 0:
                g_c = -g_c

            U, S, V = np.linalg.svd(np.dot(g_c.T, g_sample))
            M = np.dot(V.T, U.T)

            phi = np.arccos(M[2, 2]) * 180 / np.pi
            phi_2 = np.arctan2(M[2, 0], M[2, 1]) * 180 / np.pi
            phi_1 = np.arctan2(M[0, 2], -M[1, 2]) * 180 / np.pi

            t = 0
            d_dev = 0
            for r in range(0, Num - 1):
                ang_dev = np.clip(np.dot(np.dot(M, g_c[r, :]), g_sample[r, :]), -1, 1)
                d_dev = d_dev + n_c[r] * 100 * np.abs(1 / n_c[r] - dp[r])
                t = t + np.abs(np.arccos(ang_dev))
            t = t / g_sample.shape[0] * 180 / np.pi
            d_dev = d_dev / g_sample.shape[0]
            euler[sol, :] = np.array([np.around(phi_1, decimals=3), np.around(phi, decimals=3), np.around(phi_2, decimals=3), np.around(t, decimals=3), np.around(d_dev, decimals=3)], dtype=object)

        so = np.lexsort((euler[:, 4], euler[:, 3]))
        K = K[so, :, :]
        euler = euler[so]
        ui.ListBox_theo.clear()
        for h in range(0, euler.shape[0]):
            M_tilt = np.dot(rot_tilt_angle(), rotation(euler[h, 0], euler[h, 1], euler[h, 2]))
            phi_tilt = np.around(np.arccos(M_tilt[2, 2]) * 180 / np.pi, decimals=3)
            phi_2_tilt = np.around(np.arctan2(M_tilt[2, 0], M_tilt[2, 1]) * 180 / np.pi, decimals=3)
            phi_1_tilt = np.around(np.arctan2(M_tilt[0, 2], -M_tilt[1, 2]) * 180 / np.pi, decimals=3)
            ss = 'g:'
            for hh in range(0, Num - 1):
                ss = ss + ' ' + str(K[h, hh, 1:4])
            ui.ListBox_theo.addItem(ss)
            ui.ListBox_theo.addItem(str(phi_1_tilt) + ',' + str(phi_tilt) + ',' + str(phi_2_tilt))
            ui.ListBox_theo.addItem(str(euler[h, 3]) + ',' + str(euler[h, 4]))

    return K, euler

#######################################
#
# Display results . Compute the expected bands from the orientation matrix. Refine the results by rotation along x,y,z axes, or by adjusting rd and V
#
#######################################


def plot_bands():
    global B2, K, v0, Ct, pOB, width, height, image_diff, L, Dstar

    Num = np.shape(B2)[0]
    brightness()

    for z in range(0, Num - 1):

        v0 = np.array([(Ct[0] + B2[z + 1, 0] * (height - Ct[1]) - B2[z + 1, 1] * B2[z + 1, 0]) / (1 + B2[z + 1, 0]**2) - Ct[0], height - (B2[z + 1, 0] * (Ct[0] + B2[z + 1, 0] * (height - Ct[1]) - B2[z + 1, 1] * B2[z + 1, 0]) / (1 + B2[z + 1, 0]**2) + B2[z + 1, 1]) - Ct[1]])
        ds = np.sqrt(B2[z + 1, 0]**2 + 1) * B2[z + 1, 2]
        v0 = v0 * (1 + 0.5 * np.sign(v0[1]) * B2[z + 1, 2] / np.linalg.norm(v0))
        pOB = B2[z + 1, 1] - 0.5 * ds

        a.plot([-2 * Ct[0], 2 * Ct[0]], [height - (pOB + B2[z + 1, 0] * (-2) * Ct[0]), height - (pOB + B2[z + 1, 0] * 2 * Ct[0])], 'r-')
        a.plot([-2 * Ct[0], 2 * Ct[0]], [height - (pOB + ds / 2 + B2[z + 1, 0] * (-2) * Ct[0]), height - (pOB + ds / 2 + B2[z + 1, 0] * 2 * Ct[0])], 'b-')
        a.plot([-2 * Ct[0], 2 * Ct[0]], [height - (pOB - ds / 2 + B2[z + 1, 0] * (-2) * Ct[0]), height - (pOB - ds / 2 + B2[z + 1, 0] * 2 * Ct[0])], 'b-')

        a.plot([Ct[0], Ct[0] + v0[0]], [Ct[1], Ct[1] + v0[1]], 'g-')
        a.annotate(str(z), (Ct[0] + v0[0], Ct[1] + v0[1]))

    a.axis('off')
    a.figure.canvas.draw()


def plot_orientation_init():
    global M, sol, euler, angx, angy, angz, calib, lambd
    line = ui.ListBox_theo.currentRow()
    sol = int(line / 3)
    M = rotation(euler[sol, 0], euler[sol, 1], euler[sol, 2])
    angx, angy, angz = 0, 0, 0
    ui_Refine.tx_label.setText(str(np.around(angx, decimals=2)))
    ui_Refine.ty_label.setText(str(np.around(angy, decimals=2)))
    ui_Refine.tz_label.setText(str(np.around(angz, decimals=2)))
    M_tilt = np.dot(rot_tilt_angle(), M)
    phi_tilt = np.around(np.arccos(M_tilt[2, 2]) * 180 / np.pi, decimals=3)
    phi_2_tilt = np.around(np.arctan2(M_tilt[2, 0], M_tilt[2, 1]) * 180 / np.pi, decimals=3)
    phi_1_tilt = np.around(np.arctan2(M_tilt[0, 2], -M_tilt[1, 2]) * 180 / np.pi, decimals=3)
    ui_Refine.euler_label.setText(str(phi_1_tilt) + ',' + str(phi_tilt) + ',' + str(phi_2_tilt))
    plot_orientation()


def plot_orientation():
    global B2, K, v0, Ct, pOB, width, height, image_diff, L, Dstar, sol, M, calib, lambd

    brightness()
    a = plt.gca()
    g_c = K[sol, :, 1:4]
    g_c = np.dot(Dstar, g_c.T)
    n_c = np.linalg.norm(g_c.T, axis=1)
    g_c = (g_c / n_c).T
    L = calib / lambd
    for z in range(0, g_c.shape[0]):
        g_xyz = np.dot(M, g_c[z, :])
        vm = L * np.array([0, 0, 1]) - L * g_xyz[2] * g_xyz
        br = np.arcsin(lambd / 2 / distance(K[sol, z, 1], K[sol, z, 2], K[sol, z, 3]))
        vp1 = vm + np.linalg.norm(vm) * np.tan(br) * g_xyz
        vp2 = vm - np.linalg.norm(vm) * np.tan(br) * g_xyz

        a.plot([Ct[0], Ct[0] - vp1[0]], [Ct[1], Ct[1] + vp1[1]], 'g-')
        a.annotate(str(z), (Ct[0] - vp1[0], Ct[1] + vp1[1]))
        a.plot([-vp1[0] + Ct[0] - 1000 * vp1[1], -vp1[0] + Ct[0] + 1000 * vp1[1]], [vp1[1] + Ct[1] - 1000 * vp1[0], vp1[1] + Ct[1] + 1000 * vp1[0]], 'r-')
        a.plot([-vp2[0] + Ct[0] - 1000 * vp2[1], -vp2[0] + Ct[0] + 1000 * vp2[1]], [vp2[1] + Ct[1] - 1000 * vp2[0], vp2[1] + Ct[1] + 1000 * vp2[0]], 'r-')

        a.axis('off')
        a.figure.canvas.draw()


def Rxm():
    global M, angx
    tx = -np.float64(ui_Refine.Rx_entry.text())
    M = np.dot(Rot(tx, 1, 0, 0), M)
    angx = angx + tx
    update_orientation()
    return


def Rxp():
    global M, angx
    tx = np.float64(ui_Refine.Rx_entry.text())
    M = np.dot(Rot(tx, 1, 0, 0), M)
    angx = angx + tx
    update_orientation()
    return


def Rym():
    global angy, M
    ty = -np.float64(ui_Refine.Ry_entry.text())
    M = np.dot(Rot(ty, 0, 1, 0), M)
    angy = angy + ty
    update_orientation()
    return


def Ryp():
    global angy, M
    ty = np.float64(ui_Refine.Ry_entry.text())
    M = np.dot(Rot(ty, 0, 1, 0), M)
    angy = angy + ty
    update_orientation()
    return


def Rzm():
    global angz, M
    tz = np.float64(ui_Refine.Rz_entry.text())
    M = np.dot(Rot(tz, 0, 0, 1), M)
    angz = angz + tz
    update_orientation()
    return


def Rzp():
    global angz, M
    tz = -np.float64(ui_Refine.Rz_entry.text())
    M = np.dot(Rot(tz, 0, 0, 1), M)
    angz = angz + tz
    update_orientation()
    return


def update_L():
    global calib, M, angx, angy, angz, Vt
    if ui_Refine.L_entry.text() == '':
        ui_Refine.L_entry.setText(str(x_calib[ui.Calib_box.currentIndex()][4]))
    if ui_Refine.V_entry.text() == '':
        ui_Refine.V_entry.setText(str(x_calib[ui.Calib_box.currentIndex()][1]))
    calib = np.float64(ui_Refine.L_entry.text())
    Vt = np.float64(ui_Refine.V_entry.text()) * 1e3
    show_result()
    plot_orientation_init()


def update_orientation():
    global calib, Vt, M, angx, angy, angz

    calib = np.float64(ui_Refine.L_entry.text())
    Vt = np.float64(ui_Refine.V_entry.text())
    ui_Refine.tx_label.setText(str(np.around(angx, decimals=2)))
    ui_Refine.ty_label.setText(str(np.around(angy, decimals=2)))
    ui_Refine.tz_label.setText(str(np.around(angz, decimals=2)))
    M_tilt = np.dot(rot_tilt_angle(), M)
    phi = np.arccos(M_tilt[2, 2]) * 180 / np.pi
    phi_2 = np.arctan2(M_tilt[2, 0], M_tilt[2, 1]) * 180 / np.pi
    phi_1 = np.arctan2(M_tilt[0, 2], -M_tilt[1, 2]) * 180 / np.pi
    ui_Refine.euler_label.setText(str(np.around(phi_1, decimals=3)) + ',' + str(np.around(phi, decimals=3)) + ',' + str(np.around(phi_2, decimals=3)))
    plot_orientation()
    return

###################################################
#
# Get extinction from the space group
#
#####################################################


def extinction(space_group, h, k, l):
    global x_space, G
    F = 0
    s = 0
    q = 2 * np.pi * 1e-10 / (np.sqrt(np.dot(np.array([h, k, l]), np.dot(np.linalg.inv(G), np.array([h, k, l])))))

    for i in range(0, len(x_space)):
        if space_group == x_space[i][0]:
            s = i

    while (s < (len(x_space) - 1) and (len(x_space[s + 1]) == 4)):

        f = str(x_space[s + 1][0])
        for z in range(0, len(x_scatt)):
            if f == x_scatt[z][0]:
                f = eval(x_scatt[z][1]) * np.exp(-eval(x_scatt[z][2]) * (q / 4 / np.pi)**2) + eval(x_scatt[z][3]) * np.exp(-eval(x_scatt[z][4]) * (q / 4 / np.pi)**2) + eval(x_scatt[z][5]) * np.exp(-eval(x_scatt[z][6]) * (q / 4 / np.pi)**2) + eval(x_scatt[z][7]) * np.exp(-eval(x_scatt[z][8]) * (q / 4 / np.pi)**2) + eval(x_scatt[z][9])
        F = F + f * np.exp(2j * np.pi * (eval(x_space[s + 1][1]) * h + eval(x_space[s + 1][2]) * k + eval(x_space[s + 1][3]) * l))
        s = s + 1
    I = np.around(float(np.real(F * np.conj(F))), decimals=2)
    return I


######################################################################################
#
# Change the brightness of the image
# Use the mean intensity mean_ih of the gray scaled original image to scale the slider
#
#####################################################################################


def brightness():
    a.clear()
    img = Image.open(str(image_diff[0]))
    ui.brightness_slider.setMinimum(10)
    ui.brightness_slider.setMaximum(int(25500 / mean_ih))

    bv = ui.brightness_slider.value()
    if img.mode == 'P':
        img = img.convert('L')
    enhancer = ImageEnhance.Brightness(img)
    img = enhancer.enhance(bv / 100)

    figure.suptitle(str(image_diff[0]))
    a.imshow(img, origin='upper')
    a.axis([0, width, height, 0])
    a.axis('off')
    a.figure.canvas.draw()


#####################################################################################
#
# Import crystal structure from the structures.txt file: a,b,c,alpha,beta,gamma,space group
#
#####################################################################################

def structure(item):
    global x0, var_hexa, d_label_var, e_entry

    ui.abc_entry.setText(str(item[1]) + ',' + str(item[2]) + ',' + str(item[3]))
    ui.alphabetagamma_entry.setText(str(item[4]) + ',' + str(item[5]) + ',' + str(item[6]))

    ii = ui.space_group_Box.findText(str(item[7]))
    ui.space_group_Box.setCurrentIndex(ii)

##################################################
#
# Add matplotlib toolbar to zoom and pan
#
###################################################


class NavigationToolbar(NavigationToolbar):
    # only display the buttons we need
    toolitems = [t for t in NavigationToolbar.toolitems if
                 t[0] in ('Home', 'Pan', 'Zoom')]

    def set_message(self, msg):
        pass


def except_hook(cls, exception, traceback):
    sys.__excepthook__(cls, exception, traceback)

###############################################################
#
# Main, import Gui from KikuchiUI and refineUI
#
################################################################


try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


if __name__ == "__main__":

    app = QtWidgets.QApplication(sys.argv)
    sys.excepthook = except_hook
    QtWidgets.qApp.setApplicationName("Kikuchi")
    Index = QtWidgets.QMainWindow()
    ui = kikuchiUI.Ui_Kikuchi()
    ui.setupUi(Index)
    figure = plt.figure(facecolor='white', figsize=[2, 2], dpi=100)
    canvas = FigureCanvas(figure)
    a = figure.add_subplot(111)
    a.axis("off")
    ui.mplvl.addWidget(canvas)
    toolbar = NavigationToolbar(canvas, canvas)
    toolbar.setMinimumWidth(601)

    # read the txt files

    f_space = open(os.path.join(os.path.dirname(__file__), 'space_group.txt'), "r")

    x_space = []

    for line in f_space:
        x_space.append(list(map(str, line.split())))

    list_space_group = []
    for i in range(0, len(x_space)):
        if len(x_space[i]) == 1:
            list_space_group.append(x_space[i][0])

    f_space.close()

    file_struct = open(os.path.join(os.path.dirname(__file__), 'structure.txt'), "r")

    x0 = []

    for line in file_struct:
        x0.append(list(map(str, line.split())))

    i = 0
    file_struct.close()

    for item in x0:
        entry = ui.menuStructure.addAction(item[0])
        entry.triggered.connect(lambda checked, item=item: structure(item))
        ui.space_group_Box.addItem(x0[i][7])
        i = i + 1

################################################
#
# Import calibrations from the txt file: microscope name..., E(kV), r.d constant in a calibrated diffraction pattern (d should be in A, r in pixel)
#
##############################################

    f_calib = open(os.path.join(os.path.dirname(__file__), 'calibrations.txt'), "r")

    x_calib = []

    for line in f_calib:
        x_calib.append(list(map(str, line.split())))

    f_calib.close()
    counter = len(x_calib)

    for i in range(counter):
        ui.Calib_box.addItem(x_calib[i][0] + ' ' + x_calib[i][1] + 'keV ' + x_calib[i][2] + 'cm' + ', Binning:' + x_calib[i][3])

    f_scatt = open(os.path.join(os.path.dirname(__file__), 'scattering.txt'), "r")

    x_scatt = []

    for line in f_scatt:
        x_scatt.append(list(map(str, line.split())))

    f_scatt.close()

    ui.actionSave_figure.triggered.connect(open_image)
    figure.canvas.mpl_connect('button_press_event', onpress)
    figure.canvas.mpl_connect('button_release_event', onrelease)
    figure.canvas.mpl_connect('motion_notify_event', onmove)
    press = False
    move = False

    Tilt = QtWidgets.QDialog()
    ui_Tilt = tiltUI.Ui_Tilt()
    ui_Tilt.setupUi(Tilt)
    ui.actionTilt.triggered.connect(Tilt.show)
    ui_Tilt.tilt_a_entry.setText('0')
    ui_Tilt.tilt_b_entry.setText('0')
    ui_Tilt.tilt_z_entry.setText('0')
    ui_Tilt.t_ang_entry.setText('0')

    Refine = QtWidgets.QDialog()
    ui_Refine = refineUI.Ui_Refine()
    ui_Refine.setupUi(Refine)
    ui.actionRefine_orientation.triggered.connect(Refine.show)
    ui_Refine.Rxm_button.clicked.connect(Rxm)
    ui_Refine.Rxp_button.clicked.connect(Rxp)
    ui_Refine.Rym_button.clicked.connect(Rym)
    ui_Refine.Ryp_button.clicked.connect(Ryp)
    ui_Refine.Rzm_button.clicked.connect(Rzm)
    ui_Refine.Rzp_button.clicked.connect(Rzp)
    ui_Refine.update_button.clicked.connect(update_L)
    ui_Refine.Rx_entry.setText('0.1')
    ui_Refine.Ry_entry.setText('0.1')
    ui_Refine.Rz_entry.setText('0.1')

    ui.Button_reset.clicked.connect(reset_points)
    ui.reset_view_button.clicked.connect(reset)
    ui.Button_addband.clicked.connect(addband)
    ui.Button_remove_band.clicked.connect(removeband)
    ui.Button_add_center.clicked.connect(addcenter)
    ui.brightness_slider.setMinimum(1)
    ui.brightness_slider.setMaximum(50)
    ui.brightness_slider.valueChanged.connect(brightness)
    ui.plot_bands_button.clicked.connect(plot_bands)
    ui.Button_orientation.clicked.connect(plot_orientation_init)
    ui.Button_run.clicked.connect(show_result)
    ui.indice_entry.setText('3')
    ui.tol_entry.setText('1')
    s = 1
    gclick = np.zeros((1, 2))
    B2 = np.zeros((1, 3))

    Index.show()
    sys.exit(app.exec_())
